/*----------------------------------------------------------------------------*/
/*  fd.cpp                                                                    */
/*----------------------------------------------------------------------------*/
/*  copyright         : (C) 2002 by Dao Hai Lam                               */
/*                      VISC Software & Security Consultant Company           */
/*                      Hall 3, Quang Trung Software City                     */
/*                      Tan Chanh Hiep Ward, District 12,                     */
/*                      Ho Chi Minh city, VIETNAM                             */
/*  website           : http://www.visc-network.com                           */
/*  email             : daohailam at yahoo dot com                            */
/*  last modify       : Sat, 27 Sep 2003 14:31:05 +0700                       */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*   This program is free software; you can redistribute it and/or modify     */
/*   it under the terms of the GNU General Public License as published by     */
/*   the Free Software Foundation; either version 2 of the License, or        */
/*   (at your option) any later version.                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#include "fd.h"
#include "data.h"
#include "fdlookup.h"
#include "fdcontents.h"
#include "fdapplication.h"
#include "vddatabase.h"
#include "vdpreferences.h"
#include "vdictdb.h"
/*----------------------------------------------------------------------------*/
#include "fdenable.xpm"
#include "fddisable.xpm"
/*----------------------------------------------------------------------------*/
#include <ctype.h>
#include <qpixmap.h>
#include <qtooltip.h>
#include <qclipboard.h>
#include <qtimer.h>
/*----------------------------------------------------------------------------*/
#define	FD_QUIT		0x0000
#define	FD_POPUP	0x1000
#define	FD_DATA		0x2000
#define	FD_CONFIG	0x3000
#define	FD_ABOUT	0x4000
#define	FD_LOOKUP	0x5000
#define	FD_LASTWORD	0x6000
/*----------------------------------------------------------------------------*/
#define	FD_LOGO_WIDTH	32
#define	FD_LOGO_HEIGHT	32
/*----------------------------------------------------------------------------*/
QPixmap *fd_enable;
QPixmap *fd_disable;
QPopupMenu *fdMainMenu;
bool fdPopupAtMouse = true;
/*----------------------------------------------------------------------------*/
FD::FD(): QWidget( 0, 0, WStyle_Customize|WStyle_NoBorder|WDestructiveClose )
{
	ready = false;
	qApp->clipboard()->clear();
	fd_enable = new QPixmap( fdenable_xpm );
	fd_disable = new QPixmap( fddisable_xpm );
	QToolTip::add( this, "VISC Flash Dictionary" );
	
	if( Resource::last_x<0 ) Resource::last_x = 0;
	if( Resource::last_y<0 ) Resource::last_y = 0;
	if( Resource::last_x+FD_LOGO_WIDTH>=QApplication::desktop()->width() )
		Resource::last_x = QApplication::desktop()->width()-FD_LOGO_WIDTH-1;
	if( Resource::last_y+FD_LOGO_HEIGHT>=QApplication::desktop()->height() )
		Resource::last_y = QApplication::desktop()->height()-FD_LOGO_HEIGHT-1;
	setGeometry( Resource::last_x, Resource::last_y, FD_LOGO_WIDTH, FD_LOGO_HEIGHT );
	VDPreferences::setDefaultStyle( Resource::defaultStyle );
	
	fdlookup = new FDLookup();
	fdlookup->setFont( Resource::defaultFont );
	fdcontents = new FDContents();
	fdcontents->setFont();
	
	fdMenu = new QPopupMenu( 0, "FDMenu" );
	connect( fdMenu, SIGNAL( activated(int) ), SLOT( fdMenuClicked(int) ) );
	fdMenu->setCheckable(true);
	fdMenu->insertItem( "Enable", FD_POPUP );
	fdMenu->insertSeparator();
	fdMenu->insertItem( "Database...", FD_DATA );
	fdMenu->insertItem( "Configuration...", FD_CONFIG );
	fdMenu->insertSeparator();
	fdMenu->insertItem( "About...", FD_ABOUT );
	fdMenu->setItemChecked( FD_POPUP, fdPopupAtMouse );
	
	fdMainMenu = new QPopupMenu( 0, "FDMainMenu" );
	connect( fdMainMenu, SIGNAL( activated(int) ), SLOT( fdMainMenuClicked(int) ) );
	fdMainMenu->insertItem( "Flash dictionary", fdMenu );
	fdMainMenu->insertSeparator();
	for( uint i=0; i<dictList.count(); i++ ) {
		cdict = dictList.at(i);
		fdMainMenu->insertItem( QString::fromUtf8(cdict->getDescription()), (int)cdict );
	}
	if( dictList.count()>0 ) {
		cdict = dictList.at(0);
		fdMainMenu->setItemChecked( (int)cdict, true );
		fdlookup->updateInfo();
	}
	fdMainMenu->insertSeparator();
	fdMainMenu->insertItem( "Lookup...", FD_LOOKUP );
	fdMainMenu->insertItem( "Last reference", FD_LASTWORD );
	fdMainMenu->insertSeparator();
	fdMainMenu->insertItem( "Quit", FD_QUIT );
	fdMainMenu->move( -1000, -1000 );
	fdMainMenu->show();
	fdMainMenu->hide();
	
	lastWords = "";
	timer = new QTimer( this, "timer" );
	connect( timer, SIGNAL(timeout()), SLOT(checkClipboard()) );
	timer->start( 1000, false );
	ready = true;
}
/*----------------------------------------------------------------------------*/
FD::~FD()
{
	ready = false;
	timer->stop();
	delete fdcontents;
	delete fdlookup;
	delete fdMainMenu;
	delete fdMenu;
	delete fd_enable;
	delete fd_disable;
}
/*----------------------------------------------------------------------------*/
void FD::paintEvent(QPaintEvent *pe)
{
	if( ready ) {
		QPainter p(this);
		p.drawPixmap( 0, 0, fdPopupAtMouse ? *fd_enable : *fd_disable );
		p.end();
	}
}
/*----------------------------------------------------------------------------*/
void FD::mousePressEvent(QMouseEvent *me)
{
	QRect g;
	QPoint c;
	switch( me->button() ) {
		case LeftButton:
			dragging = true;
			has_moved = false;
			c = QCursor::pos();
			dx = c.x();
			dy = c.y();
			break;
		case MidButton:
			if( ready ) {
				fdlookup->popup();
			#if QT_VERSION >= 300
				QApplication::clipboard()->clear( QClipboard::Selection );
			#else
				QApplication::clipboard()->clear();
			#endif
			}
			break;
		case RightButton:
			g = geometry();
			if( g.y() + FD_LOGO_HEIGHT + fdMainMenu->height() > QApplication::desktop()->height() )
				fdMainMenu->popup(QPoint( g.x(), g.y() - fdMainMenu->height() ));
			else
				fdMainMenu->popup(QPoint( g.x(), g.y() + g.height()));
			break;
	}
}
/*----------------------------------------------------------------------------*/
void FD::mouseMoveEvent( QMouseEvent *me )
{
	if( dragging ) {
		QPoint c = QCursor::pos();

		has_moved = true;
		Resource::last_x = x() + c.x() - dx;
		Resource::last_y = y() + c.y() - dy;
		dx = c.x();
		dy = c.y();
		if( Resource::last_x<0 ) Resource::last_x = 0;
		if( Resource::last_y<0 ) Resource::last_y = 0;
		if( Resource::last_x+FD_LOGO_WIDTH>=QApplication::desktop()->width() )
			Resource::last_x = QApplication::desktop()->width()-FD_LOGO_WIDTH-1;
		if( Resource::last_y+FD_LOGO_HEIGHT>=QApplication::desktop()->height() )
			Resource::last_y = QApplication::desktop()->height()-FD_LOGO_HEIGHT-1;
		
		move( Resource::last_x, Resource::last_y );
	}
}
/*----------------------------------------------------------------------------*/
void FD::mouseReleaseEvent( QMouseEvent *me )
{
	if( me->button()==LeftButton ) {
		dragging = false;
		if( !has_moved )
			fdMenuClicked( FD_POPUP );
	}
}
/*----------------------------------------------------------------------------*/
void FD::checkClipboard()
{
	raise();
	if( !ready || !fdPopupAtMouse || fdlookup->isVisible() ) return;
#if QT_VERSION >= 300
	QString words = QApplication::clipboard()->text( QClipboard::Selection );
	QApplication::clipboard()->clear( QClipboard::Selection );
#else
	QString words = QApplication::clipboard()->text();
	QApplication::clipboard()->clear();
#endif
	words = words.simplifyWhiteSpace();
	if( !words.isEmpty() ) {
		char *w = (char *)(const char *)words;
		if( !w ) return;

		while( *w && (isspace( *w ) || ispunct( *w )) ) w++;
		if( *w ) {
			char *pp, *p = strchr(w, '\n');
			if( p ) *p = 0;
			p = pp = &w[strlen(w)-1];
			while( p!=w && (isspace( *p ) || ispunct( *p )) ) p--;
			if( p!=pp ) *(++p) = 0;
			if( *w ) {
				lastWords = QString( w );
				fdcontents->show( lastWords );
			}
		}
	}
}
/*----------------------------------------------------------------------------*/
void FD::fdMenuClicked( int id )
{
	switch( id ) {
		case FD_POPUP:
			FDClearSelection();
			fdMenu->setItemChecked( FD_POPUP, fdPopupAtMouse = !fdPopupAtMouse );
			paintEvent( NULL );
			break;
		case FD_DATA: {
				VDDatabase fddatabase;
				connect( &fddatabase, SIGNAL( updated(int) ), SLOT( updateItem(int) ) );
				connect( &fddatabase, SIGNAL( deleted(int) ), SLOT( deleteItem(int) ) );
				connect( &fddatabase, SIGNAL( changedInfo(int) ), SLOT( modifyItem(int) ) );
				connect( &fddatabase, SIGNAL( changedOrder(int,int) ), SLOT( moveItem(int,int) ) );
				fddatabase.execute();
			}
			break;
		case FD_CONFIG: {
				VDPreferences fdPreferences;
				fdPreferences.execute();
			}
			break;
		case FD_ABOUT:
			fdcontents->help();
			break;
	}
}
/*----------------------------------------------------------------------------*/
void FD::updateItem( int t )
{
	VDictDB *d = dictList.at( t );
	fdMainMenu->insertItem( QString::fromUtf8(d->getDescription()), (int)d, t+2 );
	if( !cdict ) {
		cdict = d;
		fdMainMenu->setItemChecked( (int)d, true );
	}
}
/*----------------------------------------------------------------------------*/
void FD::deleteItem( int t )
{
	fdMainMenu->removeItemAt( t + 2 );
	if( dictList.find( cdict ) < 0 ) cdict = NULL;
}
/*----------------------------------------------------------------------------*/
void FD::modifyItem( int t )
{
	VDictDB *d = dictList.at( t );
	fdMainMenu->changeItem( (int)d, QString::fromUtf8( d->getDescription() ) );
}
/*----------------------------------------------------------------------------*/
void FD::moveItem( int t, int n )
{
	VDictDB *d = dictList.at( n );
	fdMainMenu->removeItemAt( t + 2 );
	fdMainMenu->insertItem( QString::fromUtf8(d->getDescription()), (int)d, n+2 );
	if( cdict == d ) fdMainMenu->setItemChecked( (int)d, true );
}
/*----------------------------------------------------------------------------*/
void FD::fdMainMenuClicked( int id )
{
	switch( id ) {
		case FD_QUIT:
			ready = false;
			close();
			break;
		case FD_LOOKUP:
			if( ready )
				fdlookup->popup();
			break;
		case FD_LASTWORD:
			if( !lastWords.isEmpty() )
				fdcontents->show( lastWords );
			break;
		default:
			if( id>=0 && cdict!=(VDictDB *)id ) {
				fdMainMenu->setItemChecked( (int)cdict, false );
				fdMainMenu->setItemChecked( (int)(cdict = (VDictDB *)id), true );
			}
			break;
	}
}
/*----------------------------------------------------------------------------*/
