/*----------------------------------------------------------------------------*/
/*  fdcontents.cpp                                                            */
/*----------------------------------------------------------------------------*/
/*  copyright         : (C) 2002 by Dao Hai Lam                               */
/*                      VISC Software & Security Consultant Company           */
/*                      Hall 3, Quang Trung Software City                     */
/*                      Tan Chanh Hiep Ward, District 12,                     */
/*                      Ho Chi Minh city, VIETNAM                             */
/*  website           : http://www.visc-network.com                           */
/*  email             : daohailam at yahoo dot com                            */
/*  last modify       : Sat, 27 Sep 2003 14:31:05 +0700                       */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*   This program is free software; you can redistribute it and/or modify     */
/*   it under the terms of the GNU General Public License as published by     */
/*   the Free Software Foundation; either version 2 of the License, or        */
/*   (at your option) any later version.                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#include "data.h"
#include "fdlookup.h"
#include "fdcontents.h"
#include "fdapplication.h"
#include "vdictdb.h"
#include "resource.h"
#include "vdcontentviewer.h"
/*----------------------------------------------------------------------------*/
#include "mini-fd.xpm"
/*----------------------------------------------------------------------------*/
#include <stdlib.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qpushbutton.h>
#include <qapplication.h>
#include <qclipboard.h>
#include <qcursor.h>
/*----------------------------------------------------------------------------*/
FDContents::FDContents()
{
	QGridLayout *grid = new QGridLayout( this, 0, 0, 0 );
	grid->addWidget( viewer = new VDContentViewer( this ), 0, 0 );
	connect( viewer, SIGNAL( prev() ), SLOT( historyPrev() ));
	connect( viewer, SIGNAL( next() ), SLOT( historyNext() ));
	connect( viewer, SIGNAL( list() ), SLOT( historyList() ));
	connect( viewer, SIGNAL( reference(const QString&) ), SLOT( reference(const QString&) ));
	word = viewer->getWordLabel();
	resize( Resource::last_w, Resource::last_h );
}
/*----------------------------------------------------------------------------*/
FDContents::~FDContents()
{
	Resource::last_w = width();
	Resource::last_h = height();
}
/*----------------------------------------------------------------------------*/
void FDContents::show( QString w )
{
	int cx, cy, sw, sh;
	QPoint g = QCursor::pos();
	
	cx = g.x();
	cy = g.y();
	sw = qApp->desktop()->width();
	sh = qApp->desktop()->height();

	if( cx+width()>sw ) cx -= width();
	if( cy+height()>sh ) cy -= height();
	if( cx<0 ) cx = 0;
	if( cy<0 ) cy = 0;
	
#if QT_VERSION >= 0x030000
	QApplication::clipboard()->clear( QClipboard::Selection );
#else
	QApplication::clipboard()->clear();
#endif
	if( cdict ) {
		int idx = cdict->lookup( w );
		if( !cdict->isExact() ) {
			fdlookup->popup( idx );
			return;
		}
		move( cx, cy );
		word->setText( w );
		cdict->getContents( idx, contents );
		setCaption( cdict->getDescription() );
	}
	else {
		word->setText( "Warning" );
		contents->setText( fdWarningText );
	}
	QDialog::show();
}
/*----------------------------------------------------------------------------*/
void FDContents::historyList()
{
	fdlookup->popup();
}
/*----------------------------------------------------------------------------*/
void FDContents::historyPrev()
{
	if( !cdict ) return;
	if( showAbout )
		cdict->getRecent()->last( word, contents );
	else
		cdict->getRecent()->prev( word, contents );
	showAbout = false;
}
/*----------------------------------------------------------------------------*/
void FDContents::historyNext()
{
	if( cdict && !showAbout ) {
		cdict->getRecent()->next( word, contents );
		showAbout = false;
	}
}
/*----------------------------------------------------------------------------*/
void FDContents::reference( const QString& w )
{
	if( cdict ) {
		QString s = w;
		word->setText( w );
		cdict->getContents( s, contents );
		setCaption( cdict->getDescription() );
		showAbout = false;
	}
}
/*----------------------------------------------------------------------------*/
void FDContents::help()
{
	showAbout = true;
	word->setText( "About" );
	contents->setText( fdAboutText );
	setCaption( "Flash dictionary" );
	
	move( (qApp->desktop()->width() - width())/2,
			(qApp->desktop()->height() - height())/2 );
	QDialog::show();
}
/*----------------------------------------------------------------------------*/
void FDContents::setTextColor()
{
	contents->applyDefaultColor();
	fdAboutText.replace( 22, 7, Resource::defaultColors[2] );
}
/*----------------------------------------------------------------------------*/
bool FDContents::event( QEvent *e )
{
	if( e->type()==QEvent::WindowDeactivate )
		close();
	return QDialog::event( e );
}
/*----------------------------------------------------------------------------*/
void FDContents::setFont()
{
	QFont f = Resource::defaultFont;
	f.setBold( true );
	contents->setFont( Resource::defaultFont );
	word->setFont( f );
}
/*----------------------------------------------------------------------------*/
