/*----------------------------------------------------------------------------*/
/*  fdlookup.cpp                                                              */
/*----------------------------------------------------------------------------*/
/*  copyright         : (C) 2002 by Dao Hai Lam                               */
/*                      VISC Software & Security Consultant Company           */
/*                      Hall 3, Quang Trung Software City                     */
/*                      Tan Chanh Hiep Ward, District 12,                     */
/*                      Ho Chi Minh city, VIETNAM                             */
/*  website           : http://www.visc-network.com                           */
/*  email             : daohailam at yahoo dot com                            */
/*  last modify       : Sat, 27 Sep 2003 14:31:05 +0700                       */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*   This program is free software; you can redistribute it and/or modify     */
/*   it under the terms of the GNU General Public License as published by     */
/*   the Free Software Foundation; either version 2 of the License, or        */
/*   (at your option) any later version.                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#include "data.h"
#include "fdlookup.h"
#include "fdcontents.h"
#include "vdictdb.h"
#include "resource.h"
#include <qapplication.h>
#include <qlayout.h>
#include <qcursor.h>
/*----------------------------------------------------------------------------*/
#define WINDOW_SIZE		100
#define	MAX_ITEM_IDX	(WINDOW_SIZE-1)
#define vd_min(a, b)	(((a)>(b))?(b):(a))
/*----------------------------------------------------------------------------*/
bool userInput = false;
VDictionary *ldict = NULL;
/*----------------------------------------------------------------------------*/
FDLookup::FDLookup() : QDialog( 0, 0, true )
{
	resize( 270, 230 );
	setCaption( "Flash dictionary - lookup" );
	
	QBoxLayout *grid = new QVBoxLayout( this, 4 );
	grid->addWidget( edit = new VDVietEdit( this ) );
	QFrame *frame = new QFrame( this );
	frame->setFrameShape( QFrame::WinPanel );
	frame->setFrameShadow( QFrame::Sunken );
	QBoxLayout *hbox = new QHBoxLayout( frame, 2, 0 );
	sbar = new QScrollBar( QScrollBar::Vertical, frame );
	list = new VDListBox( frame );
	list->setFrameShape( QFrame::NoFrame );
	list->setHScrollBarMode( QListBox::AlwaysOff );
	list->setVScrollBarMode( QListBox::AlwaysOff );
	hbox->addWidget( list );
	hbox->addWidget( sbar );
	grid->addWidget( frame );
	
	setFont( Resource::defaultFont );
	connect( edit, SIGNAL( keyPressed() ), SLOT( lookup() ));
	connect( edit, SIGNAL( browse(int) ), SLOT( browse(int) ));
	connect( edit, SIGNAL( returnPressed() ), SLOT( reference() ));
	connect( list, SIGNAL( selected(int) ), SLOT( reference(int) ));
	connect( list, SIGNAL( highlighted(int) ), SLOT( highlighted(int) ));
	connect( list, SIGNAL( switchFocus(QKeyEvent *) ), edit, SLOT( acceptFocus(QKeyEvent *) ));
	connect( sbar, SIGNAL( valueChanged( int ) ), SLOT( itemChanged( int ) ) );
	onLookup = false;
}
/*----------------------------------------------------------------------------*/
FDLookup::~FDLookup()
{
}
/*----------------------------------------------------------------------------*/
void FDLookup::popup( int index )
{
	if( !cdict ) return;

	int cx, cy, sw, sh;
	QPoint g = QCursor::pos();
	
	cx = g.x();
	cy = g.y();
	sw = qApp->desktop()->width();
	sh = qApp->desktop()->height();

	if( cx+width()>sw ) cx -= width();
	if( cy+height()>sh ) cy -= height();
	if( cx<0 ) cx = 0;
	if( cy<0 ) cy = 0;
	move( cx, cy );
	
	if( ldict!=cdict )
		updateInfo();
	if( index>=0 )
		sbar->setValue( index );
	edit->setFocus();
	exec();
}
/*----------------------------------------------------------------------------*/
void FDLookup::itemChanged( int idx )
{
	int i;
	
	userInput = true;
	if( idx<topItem )
	{
		if( idx<topItem-WINDOW_SIZE/2 )
		{
			list->clear();
			for( i=idx; i<idx+WINDOW_SIZE; i++ )
				list->insertItem( QString::fromUtf8(cdict->getWord(i)) );
		}
		else
		for( i=topItem-1; i>=idx; i-- )
		{
			list->removeItem( MAX_ITEM_IDX );
			list->insertItem( QString::fromUtf8(cdict->getWord(i)), 0 );
		}
		topItem = idx;
	}
	else
	if( idx>=topItem+WINDOW_SIZE )
	{
		if( idx>topItem+WINDOW_SIZE+WINDOW_SIZE/2 )
		{
			list->clear();
			for( i=idx-MAX_ITEM_IDX; i<=idx; i++ )
				list->insertItem( QString::fromUtf8(cdict->getWord(i)) );
		}
		else
		for( i=topItem+WINDOW_SIZE; i<=idx; i++ )
		{
			list->removeItem( 0 );
			list->insertItem( QString::fromUtf8(cdict->getWord(i)) );
		}
		topItem = idx - MAX_ITEM_IDX;
	}
	list->setCurrentItem( idx-topItem );
	if( !onLookup )//&& c!=topItem+list->currentItem() )
	{
		edit->setText( list->currentText() );
		edit->selectAll();
	}
	userInput = false;
}
/*----------------------------------------------------------------------------*/
// Input into lookup field
void FDLookup::lookup()
{
	onLookup = true;
	sbar->setValue( cdict->lookup( edit->text() ) );
	onLookup = false;
}
/*----------------------------------------------------------------------------*/
// Press key up, down, page up, page down
void FDLookup::browse( int code )
{
	switch( code )
	{
		case Key_PageUp:
			sbar->subtractPage();
			break;
		case Key_Up:
			sbar->subtractLine();
			break;
		case Key_Down:
			sbar->addLine();
			break;
		case Key_PageDown:
			sbar->addPage();
	}
}
/*----------------------------------------------------------------------------*/
// Click on item in word list
void FDLookup::highlighted(int index)
{
	if( !userInput )
		sbar->setValue( topItem+index );
}
/*----------------------------------------------------------------------------*/
// Double click on item in word list
void FDLookup::reference( int index )
{
	hide();
	fdcontents->show( list->currentText() );
}
/*----------------------------------------------------------------------------*/
// Press enter on item in word list
void FDLookup::reference()
{
	hide();
	fdcontents->show( list->currentText() );
}
/*----------------------------------------------------------------------------*/
// Change dict
void FDLookup::updateInfo()
{
	if( !cdict ) return;
	userInput = true;
	
	list->clear();
	int n = cdict->getWordCount();
	sbar->setMaxValue( n-1 );
	n = vd_min(WINDOW_SIZE, n);
	for( int i=topItem=0; i<n; i++ )
		list->insertItem( QString::fromUtf8(cdict->getWord(i)) );
	list->setCurrentItem( -1 );
	edit->setText( "" );
	if( cdict->isLatin() )
		edit->setLatinMode();
	ldict = cdict;
	
	userInput = false;
}
/*----------------------------------------------------------------------------*/
// Process event
bool FDLookup::event( QEvent *e )
{
	if( e->type()==QEvent::WindowDeactivate )
		close();
	return QDialog::event( e );
}
/*----------------------------------------------------------------------------*/
