/*--------------------------------------------------------------------------*/
/*                       vdictdb.h  -  description                          */
/*                           -------------------                            */
/*  begin                : Fri Aug 08 2001                                  */
/*  copyright            : (C) 2000 by Dao Hai Lam                          */
/*                         VISC Software & Security Consultant Co., Ltd.    */
/*                         Hall 3, Quang Trung Software City                */
/*                         Tan Chanh Hiep Ward, District 12                 */
/*                         Ho Chi Minh city, VIETNAM                        */
/*  website              : http://www.visc-network.com                      */
/*  email                : lam@visc-network.com                             */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*   This program is free software; you can redistribute it and/or modify   */
/*   it under the terms of the GNU General Public License as published by   */
/*   the Free Software Foundation; either version 2 of the License, or      */
/*   (at your option) any later version.                                    */
/*                                                                          */
/*--------------------------------------------------------------------------*/
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <qfont.h>
#include <qlabel.h>
#include <qbuffer.h>
#include <qcstring.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qsemimodal.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qapplication.h>
#include "vdconsts.h"
#include "resource.h"
#include "vdictdb.h"
#include "utf.h"
/*--------------------------------------------------------------------------*/
#define VD_HEADER_SIZE		(VD_SIGNATURE_SIZE+sizeof(long)+sizeof(latin)+sizeof(wordCount))
/*--------------------------------------------------------------------------*/
#ifndef _ZLIB_H
#define	gzFile				FILE*
#define	gzopen(fn, m)		fopen(fn, m)
#define	gzread(fp, b, n)	fread(b, n, 1, fp)
#define	gzwrite(fp, b, n)	fwrite(b, n, 1, fp)
#define	gzclose(fp)			fclose(fp)
#define	gzseek(fp, ofs, m)	fseek(fp, ofs, m)
#define	gzgets(fp, b, n)	fgets(b, n, fp)
#define	gzeof(fp)			feof(fp)
#define	gztell(fp)			ftell(fp)
#define	gzrewind(fp)		rewind(fp)
#endif
/*--------------------------------------------------------------------------*/
char *INDEX_FILE_NOT_FOUND = "<CENTER><H1>Index file not found</H1><HR></CENTER>";
/*--------------------------------------------------------------------------*/
class VDWordCounting: public QSemiModal
{
private:
	QLabel *wordCount;
	QPushButton *cancelBtn;
public:
	VDWordCounting(): QSemiModal( 0, 0, true )
	{
		resize( 290, 70 );
		setFixedSize( 290, 70 );
		setCaption( "VISC dictionary" );
		QLabel *l = new QLabel( this );
		l->setGeometry( 10, 10, 92, 17 );
		l->setText( "Indexing..." );
		l->setAutoResize( true );
		l = new QLabel( this );
		l->setGeometry( 10, 40, 88, 17 );
		l->setText( "Word count:" );
		l->setAutoResize( true );
		wordCount= new QLabel( this );
		wordCount->setGeometry( 110, 40, 40, 20 );
		wordCount->setAutoResize( true );
		cancelBtn= new QPushButton( this );
		cancelBtn->setGeometry( 180, 30, 100, 30 );
		cancelBtn->setText( "&Cancel" );
		connect( cancelBtn, SIGNAL( clicked() ), this, SLOT( hide() ));
		show();
	}
	void setCount(int count)
	{
		QString c;
		c.setNum( count, 10 );
		wordCount->setText( c );
		qApp->processEvents();
	}
};
/*--------------------------------------------------------------------------*/
class VDIndexName : public QDialog
{
protected:
	QButtonGroup *group;
	QRadioButton *defaultBtn;
	QLineEdit *defaultName;
	QRadioButton *userSetBtn;
	QLineEdit *userSetName;
	QLabel *inputLabel;
	QComboBox *inputMethodList;
	QPushButton *okBtn;
	QPushButton *cancelBtn;
public:
	VDIndexName() : QDialog( 0, 0, true )
	{
		resize( 400, 190 );
		setFixedSize( 400, 190 );
		setCaption( "VISC dictionary" );
		group= new QButtonGroup( this );
		group->setGeometry( 10, 10, 380, 130 );
		defaultBtn= new QRadioButton( group );
		defaultBtn->setGeometry( 10, 10, 80, 30 );
		defaultBtn->setText( "Default:" );
		defaultBtn->setChecked( true );
		defaultName= new QLineEdit( group );
		defaultName->setGeometry( 100, 10, 270, 30 );
		defaultName->setReadOnly( true );
		userSetBtn= new QRadioButton( group );
		userSetBtn->setGeometry( 10, 50, 90, 30 );
		userSetBtn->setText( "User set:" );
		userSetName= new QLineEdit( group );
		userSetName->setGeometry( 100, 50, 270, 30 );
		inputLabel= new QLabel( "Input method:", group );
		inputLabel->setGeometry( 10, 90, 100, 30 );
		inputMethodList= new QComboBox( group );
		inputMethodList->setGeometry( 120, 90, 250, 30 );
		inputMethodList->insertItem( "Latin (VIQR)" );
		inputMethodList->insertItem( "Vietnamese" );
		okBtn = new QPushButton( "&OK", this );
		okBtn->setGeometry( 90, 150, 100, 30 );
		okBtn->setDefault( true );
		cancelBtn= new QPushButton( "&Cancel", this );
		cancelBtn->setGeometry( 210, 150, 100, 30 );
		connect( okBtn, SIGNAL( clicked() ), SLOT( accept() ));
		connect( cancelBtn, SIGNAL( clicked() ), SLOT( reject() ));
	}
	char *select(char *defname, char *usrname)
	{
		defaultName->setText( defname );
		userSetName->setText( usrname );
		if( QDialog::exec()==QDialog::Rejected )
		{
			free( defname );
			return NULL;
		}
		if( defaultBtn->isChecked() || userSetName->text().simplifyWhiteSpace().isEmpty() )
			return defname;
		free( defname );
		return strdup( userSetName->text().simplifyWhiteSpace() );
	}
	int isVietInput()
	{
		return inputMethodList->currentItem()==1;
	}
	int isLatinInput()
	{
		return inputMethodList->currentItem()==0;
	}
};
/*--------------------------------------------------------------------------*/
QList<VDictDB> dictList;
/*--------------------------------------------------------------------------*/
VDictDB::VDictDB( const char *findex, const char *fdata, const char *info )
{
	wordCount = 0;
	datafile = strdup( fdata );
	indexfile = strdup( findex );
	description = strdup( info );
	if( !(dfp = gzopen(fdata, "r")) )
		status = VD_DATA_NOT_FOUND;
	else
	{
		gzseek(dfp, 0, SEEK_END);
		dbsize = gztell(dfp);
		gzrewind(dfp);
		loadIndex();
	}
	current = -1;
}
/*--------------------------------------------------------------------------*/
VDictDB::~VDictDB()
{
	if( datafile ) free( datafile );
	if( indexfile ) free( indexfile );
	if( description ) free( description );
	if( dfp ) gzclose( dfp );
	if( ifp ) gzclose( ifp );
}
/*--------------------------------------------------------------------------*/
bool VDictDB::checkCRC( unsigned long crc )
{
	return true;
}
/*--------------------------------------------------------------------------*/
void VDictDB::loadIndex()
{
	unsigned long crc;
	char signature[ VD_SIGNATURE_SIZE ];
	
	ifp = gzopen( indexfile, "rb" );
	if( !ifp )
	{
		status = VD_INDEX_NOT_FOUND;
		return;
	}
	gzread( ifp, signature, VD_SIGNATURE_SIZE );
	if( strncmp( signature, VD_SIGNATURE, VD_SIGNATURE_SIZE ) )
	{
		status = VD_INDEX_BAD_FORMAT;
		return;
	}
	gzread( ifp, &crc, sizeof( crc ) );
	if( !checkCRC( crc ) )
	{
		status = VD_INDEX_BAD_FORMAT;
		return;
	}
	gzread( ifp, &latin, sizeof( latin ) );
	gzread( ifp, &wordCount, sizeof( wordCount ) );
	status = VD_READY;
}
/*--------------------------------------------------------------------------*/
long VDictDB::getIndex( long n )
{
	gzseek( ifp, VD_HEADER_SIZE + n * sizeof(long), SEEK_SET );
	gzread( ifp, &n, sizeof( n ) );
	return n;
}
/*--------------------------------------------------------------------------*/
char *VDictDB::getWord( long n )
{
	int l;
	char *w = buffer, *p;

	gzseek( dfp, getIndex( n ), SEEK_SET );
	gzgets( dfp, w, sizeof( buffer ) );
	w[ l=strlen( w )-1 ] = 0;
	if( (p = strstr( w, " /" ))!=NULL ) *p = 0;
	p = &w[l-1];
	while( isspace( *p ) ) p--;
	*(p+1) = 0;
	w++;
	while( isspace( *w ) ) w++;
	return match = w;
}
/*--------------------------------------------------------------------------*/
bool VDictDB::getContents( QString word, QTextBrowser *text )
{
	if( !isReady() )
	{
		text->setText( INDEX_FILE_NOT_FOUND );
		return true;
	}
	
	long idx = lookup( word.lower() );
	if( recent->add( idx, match ) )
		text->setText( getContents( idx ) );
	return true;
}
/*--------------------------------------------------------------------------*/
bool VDictDB::getContents(long i, QTextBrowser *text)
{
	if( !isReady() )
	{
		text->setText( INDEX_FILE_NOT_FOUND );
		return true;
	}
	
	if( recent->add( i, getWord( i ) ) )
		text->setText( getContents( i ) );
	return true;
}
/*--------------------------------------------------------------------------*/
char *VDictDB::getRawData( long i )
{
	if( !isReady() ) return "";

	if( i>=0 && i<wordCount )
	{
		char *dbuf;
		long size, idx = getIndex( i );

		if( i+1<wordCount )
			size = getIndex( i+1 ) - idx;
		else
			size = dbsize - idx;

		dbuf = new char[ size+1 ];
		gzseek( dfp, idx, SEEK_SET );
		gzread( dfp, dbuf, size );
		dbuf[ size ] = 0;
		return dbuf;
	}
	return "";
}
/*--------------------------------------------------------------------------*/
char *VDictDB::getRawData( QString word )
{
	if( !isReady() ) return "";

	long idx = lookup( word.lower() );
	return getRawData( idx );
}
/*--------------------------------------------------------------------------*/
#define S_NONE			0
#define S_ITEM			1
#define S_IDIOM			2
#define D_ITEM(a)		"<H3>"VD_ITEM_TAG"<FONT COLOR=\"%s\">%s</FONT></H3>", Resource::defaultColors[1], a
#define D_SPEECH(a)		"<H3>"VD_SPEECH_TAG"<FONT COLOR=\"%s\">[%s]</FONT></H3>", Resource::defaultColors[0], a
#define D_OTHERS(a,b)	"<H3>"VD_OTHERS_TAG"<FONT COLOR=\"%s\">%s</FONT><BR>"VD_SUB_TAG"<FONT COLOR=\"%s\">[%s]</FONT></H3>",Resource::defaultColors[1],a,Resource::defaultColors[0],b
#define D_IDIOMS()		"<H3>"VD_IDIOMS_TAG"<FONT COLOR=\"%s\">Idioms </FONT></H3><OL>", Resource::defaultColors[0]
/*--------------------------------------------------------------------------*/
static char line[4069];
/*--------------------------------------------------------------------------*/
QString VDictDB::highlight(char *p)
{
	QString s;
	char c=0;

	while( p )
	{
		char cc = 0, *sp, *pp;
		
		sp = strchr(p, ' ');
		if( sp )
		{
			cc = *(sp-1);
			if( cc=='$' ) *(sp-1) = 0;
			*sp++ = 0;
		}
		if( c=='$' )
		{
			while( (pp = strchr(p, '$'))!=NULL )
				strcpy( pp, pp+1 );
			while( (pp = strchr(p, '_'))!=NULL ) *pp = ' ';
			sprintf( line, VD_LINK_TAG"<FONT COLOR=\"%s\">%s</FONT> ", Resource::defaultColors[3], p );
			s += line;
			if( sp && *sp=='$' ) sp++;
		}
		else
		{
			if( *p=='$' ) p++;
			while( (pp = strchr(p, '_'))!=NULL ) *pp = ' ';
			char z = tolower(*p);
			pp = ((z=='c'||z=='d'||z=='l'||z=='s') && p[1]=='\'') ? p+2 : p;
			sprintf( line, "<A HREF=\"%s\">%s</A> ", pp, p );
			s += line;
		}
		c = cc;
		p = sp;
	}
	return s;
}
/*--------------------------------------------------------------------------*/
QString VDictDB::getContents(long i)
{
	if( i<wordCount )
	{
		char *p;
		int session = S_NONE;
		QCString data;

		data.sprintf( "<HTML><BODY>" );
		gzseek( dfp, getIndex( i ), SEEK_SET );
		gzgets( dfp, buffer, sizeof( buffer ) );
		buffer[strlen( buffer )-1] = 0;
		if( (p = strstr( buffer, " /" ))!=NULL )
		{
			p = strtok(p, "/");
			p = strtok(NULL, "/");
			sprintf( line, D_SPEECH( p ) );
			data += line;
			if( (p = strtok(NULL, ":"))!=NULL )
			{
				sprintf( line, "<H3>%s:</H3>", p );
				data += line;
				p = strtok(NULL, "/");
				while( p )
				{
					p = strchr(p, '$') + 1;
					*strchr(p, ')') = 0;
					sprintf( line, D_OTHERS( p, strtok(NULL, "/") ) );
					data += line;
					p = strtok(NULL, "/");
				}
			}
		}
		while( !gzeof( dfp ) )
		{
			char c, *pbuffer = &buffer[1];
				
			gzgets( dfp, buffer, sizeof( buffer ) );
			c = buffer[0];
			if( c=='@' ) break;
			buffer[strlen( buffer )-1] = 0;
			switch( c )
			{
				case '*':
					if( session==S_IDIOM )
						data += "</UL></OL>";
					else
					if( session==S_ITEM )
						data += "</UL>";
					session = S_ITEM;
					if( (p = strchr(pbuffer, '$'))!=NULL ) *p = '-';
					sprintf( line, D_ITEM( pbuffer ) );
					data += line;
					data += "<UL>";
					break;
				case '-':
					if( session==S_NONE )
						sprintf( line, "- %s<BR> ", pbuffer );
					else
					{
						if( (p = strchr(pbuffer, '$'))!=NULL ) *p++ = 0;
						if( !p )
							sprintf( line, "<LI>%s<BR> ", pbuffer );
							//sprintf( line, "<LI>%s ", pbuffer );
						else
						{
							char *pp = strchr(p, '$');
							if( !pp )
							{
								int z = sprintf( line, "<LI>%s", pbuffer);
								pp = strtok( p, "," );
								while( pp )
								{
									z += sprintf( &line[z], "<A HREF=\"%s\">%s</A>,", pp, pp );
									pp = strtok( NULL, "," );
								}
								line[z-1] = 0;
								sprintf( &line[z], "<BR> " );
							}
							else
							{
								while( pp-1>p && ispunct(*(pp-1)) )
								{
									*pp = *(pp-1);
									pp--;
								}
								*pp++ = 0;
								sprintf( line, "<LI>%s<A HREF=\"%s\">%s</A>%s<BR> ", pbuffer, p, p, pp );
								//sprintf( line, "<LI>%s<A HREF=\"%s\">%s</A>%s ", pbuffer, p, p, pp );
							}
						}
					}
					data += line;
					break;
				case '=':
					c = 0;
					data += highlight( strtok( &pbuffer[1], "+" ) ) + "<BR>";
					sprintf( line, "<EM>%s</EM><BR>", strtok( NULL, "\0" ) );
					data += line;
					break;
				case '!':
					data += "</UL>";
					if( session!=S_IDIOM )
					{
						session = S_IDIOM;
						sprintf( line, D_IDIOMS() );
						data += line;
					}
					data += "<LI>" + highlight( &pbuffer[1] ) + "<UL>";
					break;
			}
		}
		data += "</UL>";
		if( session==S_IDIOM ) data += "</OL>";
		data += "</FONT></BODY></HTML>";
		return QString::fromUtf8( (const char*)data );
	}
	return "<CENTER><H2>Word not found!</H2></CENTER><HR>";
}
/*--------------------------------------------------------------------------*/
long VDictDB::lookup(QString word)
{
	if( !isReady() ) return -1;
	
	long i, c, l = 0, r = wordCount-1;

	word = word.lower();
	if( latin )
		word = word.latin1();

	if( current>=0 )
	{
		c = ustrcmp( word, QString::fromUtf8( getWord( current ) ), latin );
		if( exact=c==0 ) return current;
		if( c<0 )
			r = current;
		else
			l = current;
	}

	while( 1 )
	{
		if( r-l<2 )
		{
			i = ustrcmp( word, QString::fromUtf8( getWord( l ) ), latin );
			exact = i == 0;
			i = i>0 ? r : l;
			break;
		}
		if( exact=(c=ustrcmp( word, QString::fromUtf8( getWord(i=(l+r)/2) ), latin ))==0 ) break;
		if( c<0 )
			r = i;
		else
			l = i;
	}
	return current = i;
}
/*--------------------------------------------------------------------------*/
void VDictDB::saveResource( char* fn )
{
	FILE *fp = fopen( fn, "a+" );
	if( fp )
	{
		fprintf( fp, "<VD %s:%s:%s>\n", datafile, indexfile, description );
		fclose( fp );
	}
}
/*--------------------------------------------------------------------------*/
void VDictDB::buildIndex()
{
	if( status==VD_INDEX_NOT_FOUND || status==VD_INDEX_BAD_FORMAT )
	{
		gzFile df;
		VDIndexName idxName;
		char buffer[256], *defname, *p;
		long n, count, crc = 0;

		defname = (char *)malloc( strlen(datafile)+8 );
		strcpy( defname, datafile );
		p = strrchr( defname, '.' );
		if( p )
			strcpy( p, ".idx" );
		else
			strcat( defname, ".idx" );
		indexfile = idxName.select( defname, indexfile );
		latin = !idxName.isVietInput();
		if( !indexfile ) return;
		if( !(df = gzopen( indexfile, "w+" )) )
		{
			QMessageBox::information( 0, "VDictionary", "Create index file failed!" );
			status = VD_CREATE_INDEX_FAILED;
			return;
		}

		p = buffer;
		n = count = 0;
		gzrewind( dfp );
		gzwrite( df, (void *)VD_SIGNATURE, VD_SIGNATURE_SIZE );
		gzwrite( df, &crc, sizeof( crc ) );
		gzwrite( df, &latin, sizeof( latin ) );
		gzwrite( df, &count, sizeof( count ) );
		
		VDWordCounting wc;
		while( gzgets( dfp, p, sizeof( buffer ) ) )
		{
			long l = strlen( p );
			if( *p=='@' )
			{
				gzwrite( df, &n, sizeof( n ) );
				wc.setCount( ++count );
			}
			n += l;
		}
		if( !wc.isVisible() )
		{
			gzclose( df );
			unlink( indexfile );
			return;
		}
		gzrewind( dfp );
		gzseek( df, VD_SIGNATURE_SIZE+sizeof( crc )+sizeof( latin ), SEEK_SET );
		gzwrite( df, &count, sizeof( count ) );
		gzclose( df );
		loadIndex();
	}
}
/*--------------------------------------------------------------------------*/
void VDictDB::create(const char *findex, const char *fdata, const char *info)
{
	QString msg;
	VDictDB *vdict;

	if( !info || *info==0 )
	{
		info = strrchr( fdata, '/' );
		if( !info )
			info = fdata;
		else
			info++;
	}
	vdict = new VDictDB( findex, fdata, info );
	switch( vdict->getStatus() )
	{
		case VD_DATA_NOT_FOUND:
			msg.sprintf("Data file %s not found !\n"
						"Please check and try again !", fdata );
			QMessageBox::information( 0, VD_TITLE, msg );
			delete vdict;
			return;
		case VD_INDEX_NOT_FOUND:
		case VD_INDEX_BAD_FORMAT:
			msg.sprintf( "Index file %s not found or incorrect format\n"
						 "Do you want to build it ?", findex );
			if( QMessageBox::warning( 0, VD_TITLE, msg, "&Yes", "&No", 0, 0, 1 )==1 )
			{
				delete vdict;
				return;
			}
			vdict->buildIndex();
			if( vdict->getStatus()!=VD_READY )
			{
				delete vdict;
				return;
			}
			break;
		case VD_NOT_ENOUGH_MEMORY:
			msg.sprintf("Not enough memory to load %s !", fdata );
			QMessageBox::information( 0, VD_TITLE, msg );
			delete vdict;
			return;
	}
	dictList.append(vdict);
}
/*--------------------------------------------------------------------------*/


