/*----------------------------------------------------------------------------*/
/*  resource.cpp                                                              */
/*----------------------------------------------------------------------------*/
/*  copyright         : (C) 2002 by Dao Hai Lam                               */
/*                      VISC Software & Security Consultant Company           */
/*                      Hall 3, Quang Trung Software City                     */
/*                      Tan Chanh Hiep Ward, District 12,                     */
/*                      Ho Chi Minh city, VIETNAM                             */
/*  website           : http://www.visc-network.com                           */
/*  email             : daohailam at yahoo dot com                            */
/*  last modify       : Sat, 27 Sep 2003 14:31:05 +0700                       */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*   This program is free software; you can redistribute it and/or modify     */
/*   it under the terms of the GNU General Public License as published by     */
/*   the Free Software Foundation; either version 2 of the License, or        */
/*   (at your option) any later version.                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
/*----------------------------------------------------------------------------*/
#include "vdconsts.h"
#include "visckey.h"
#include "resource.h"
#include "vdictdb.h"
#include "vdscan.h"
#include "bg.xpm"
/*----------------------------------------------------------------------------*/
#include <qdir.h>
#include <qmessagebox.h>
#include <qfontdatabase.h>
/*----------------------------------------------------------------------------*/
int Resource::last_x = 0;
int Resource::last_y = 0;
int Resource::last_w = 600;
int Resource::last_h = 400;
/*----------------------------------------------------------------------------*/
int Resource::defaultStyle = 0;
/*----------------------------------------------------------------------------*/
char Resource::defaultColors[5][8] = {
	"#880000",
	"#008800",
	"#000000",
	"#0000FF",
	"#000000"
};
/*----------------------------------------------------------------------------*/
QString Resource::config;
QString Resource::vdHome;
QString Resource::vdDataDir;
/*----------------------------------------------------------------------------*/
QFont Resource::defaultFont( "arial", 12, QFont::Normal, false );
/*----------------------------------------------------------------------------*/
char *Resource::defaultBackground = NULL;
QPixmap *Resource::viscBackground = NULL;
/*----------------------------------------------------------------------------*/
QStringList Resource::dbFonts;
/*----------------------------------------------------------------------------*/
void Resource::init(char *conf)
{
	QDir vdir;
	vdHome = vdir.homeDirPath() + "/.vdict";
	vdDataDir = vdHome + "/data";
	config = vdHome + "/" + conf;

	if( !vdir.exists( vdHome ) )
		vdir.mkdir( vdHome );
	if( !vdir.exists( vdDataDir ) )
		vdir.mkdir( vdDataDir );

	QFontDatabase db;
	dbFonts = db.families();

	if( dbFonts.isEmpty() ) {
		QMessageBox mb( "VISC Dictionary",
						"For using this software!\n"
						"Please install some unicode fonts supporting Vietnamese",
						QMessageBox::Information,
						QMessageBox::Ok,
						QMessageBox::NoButton,
						QMessageBox::NoButton );
		mb.exec();
		exit(1);
	}
	viscBackground = new QPixmap( (const char** )bg_xpm );
	dictList.setAutoDelete( true );
}
/*----------------------------------------------------------------------------*/
void Resource::load(char *conf)
{
	FILE *fp;
	char buffer[256], *p, *datafile, *indexfile;

	init(conf);	
	fp = fopen( config, "r" );
	if( !fp ) {
		VDScanner scanner( 0 );
		scanner.execute();
		return;
	}
	while( fgets( buffer, sizeof( buffer ), fp ) ) {
		p = &buffer[ strlen( buffer )-1 ];
		if( *p != '\n' ) {
			while( fgets(buffer, sizeof( buffer ), fp) )
				if( buffer[ strlen( buffer )-1 ] == '\n' ) break;
			continue;
		}
		while( isspace( *p ) ) p--;
		*(p+1) = 0;
		p = buffer;
		while( isspace( *p ) ) p++;
		if( *p == '#' || *p == 0 ) continue;
		
		datafile = NULL;
		indexfile = NULL;
		if( !strncmp( p, "<XY ", 4 ) ) {
			sscanf(p+4, "%d %d %d %d", &last_x, &last_y, &last_w, &last_h);
			if( last_x < 0 ) last_x = 0;
			if( last_y < 0 ) last_y = 0;
			if( last_w < 0 ) last_w = 640;
			if( last_h < 0 ) last_w = 480;
		}
		else
		if( !strncmp( p, "<FONT ", 6 ) ) {
			char *family = strtok( p+6, ":" );
			int size = atoi( strtok( NULL, ">" ) );
		#if QT_VERSION >= 300
			defaultFont = QFont( family, size, QFont::Normal, false );
		#else
			defaultFont = QFont( family, size, QFont::Normal, false, QFont::Unicode );
		#endif
		}
		else
		if( !strncmp( p, "<COLOR ", 7 ) ) {
			char *color = strtok( p+7, ":" );
			for( int i = 0; i < 4 && color != NULL; i++ ) {
				while( isspace( *color ) ) color++;
				strncpy( defaultColors[i], color, 7 );
				color = strtok( NULL, ":" );
			}
			strcpy( defaultColors[4], defaultColors[2] );
		}
		else
		if( !strncmp( p, "<BACKGROUND ", 12 ) ) {
			defaultBackground = strdup( strtok( p+12, ">" ) );
		}
		else
		if( !strncmp( p, "<STYLE ", 7 ) ) {
			defaultStyle = atoi( strtok( p+7, ">" ) ) % 8;
		}
		else
		if( !strncmp( p, "<METHOD ", 8 ) )
			visckey = atoi( p+8 ) % 3;
		else
		if( !strncmp( p, "<VD ",4 ) ) {
			if( (p = strtok(p+4, ":"))!=NULL ) {
				while( isspace( *p ) ) p++;
				datafile = p;
				if( (p = strtok(NULL, ">")) != NULL && *p ) {
					while( isspace( *p ) ) p++;
					if( *p ) indexfile = p;
				}
			}
			if( datafile && *datafile && indexfile && *indexfile )
				VDictDB::load( indexfile, datafile );
		}
	}
	fclose( fp );
#if QT_VERSION >= 300
	defaultFont.setStyleStrategy( QFont::PreferAntialias );
#endif
}
/*----------------------------------------------------------------------------*/
void Resource::save()
{
	FILE *fp = fopen( config, "w+" );
	if( fp ) {
		fprintf( fp, "# VISC Dictionary (VD) configuration file.\n" );
		fprintf( fp, "# This file will be updated automatically after each session of VD.\n" );
		fprintf( fp, "# Entry format:\n#\t<VD datafile:indexfile>\n" );
		fprintf( fp, "<XY %d %d %d %d>\n", last_x, last_y, last_w, last_h );
		fprintf( fp, "<METHOD %d>\n", visckey );
		fprintf( fp, "<STYLE %d>\n", defaultStyle );
		fprintf( fp, "<FONT %s:%d>\n", (const char *)defaultFont.family(), defaultFont.pointSize() );
		fprintf( fp, "<COLOR %s:%s:%s:%s>\n", defaultColors[0], defaultColors[1], defaultColors[2], defaultColors[3] );
		if( defaultBackground ) {
			fprintf( fp, "<BACKGROUND %s>\n", defaultBackground );
			free( defaultBackground );
		}
		fclose( fp );
	}
	for( uint i = 0; i < dictList.count(); i++ )
		dictList.at( i )->saveResource( (char *)(const char *)config );
	dictList.setAutoDelete( false );

	delete viscBackground;
}
/*----------------------------------------------------------------------------*/
void Resource::setBackground( int mode, QColor color, QString pixmap )
{
	if( defaultBackground )
		free( defaultBackground );

	switch( mode ) {
		case 1:
			defaultBackground = strdup(QString("C"+color.name()));
			break;
		case 2:
			defaultBackground = strdup(QString("F"+pixmap.stripWhiteSpace()));
		default:
			defaultBackground = VD_DEFAULT_BACKGROUND;
			break;
	}
}
/*----------------------------------------------------------------------------*/
QBrush Resource::getBgPaper()
{
	QBrush paper;
	if( defaultBackground == VD_DEFAULT_BACKGROUND )
		paper.setPixmap( *viscBackground );
	else
	switch( *defaultBackground ) {
		case 'C':
			paper.setColor( QColor( &defaultBackground[1] ) );
			paper.setStyle( Qt::SolidPattern );
			break;
		case 'F':
			paper.setPixmap( QPixmap( QString( &defaultBackground[1] ) ) );
			break;
		default:
			paper.setPixmap( *viscBackground );
			break;
	}
	return paper;
}
/*----------------------------------------------------------------------------*/
int Resource::getBgMode()
{
	if( defaultBackground == VD_DEFAULT_BACKGROUND )
		return 0;
	if( *defaultBackground == 'C' )
		return 1;
	return 2;
}
/*----------------------------------------------------------------------------*/
