/*----------------------------------------------------------------------------*/
/*  vddatabase.cpp                                                            */
/*----------------------------------------------------------------------------*/
/*  copyright         : (C) 2002 by Dao Hai Lam                               */
/*                      VISC Software & Security Consultant Company           */
/*                      Hall 3, Quang Trung Software City                     */
/*                      Tan Chanh Hiep Ward, District 12,                     */
/*                      Ho Chi Minh city, VIETNAM                             */
/*  website           : http://www.visc-network.com                           */
/*  email             : daohailam at yahoo dot com                            */
/*  last modify       : Sat, 27 Sep 2003 14:31:05 +0700                       */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*   This program is free software; you can redistribute it and/or modify     */
/*   it under the terms of the GNU General Public License as published by     */
/*   the Free Software Foundation; either version 2 of the License, or        */
/*   (at your option) any later version.                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#include <vdconsts.h>
#include <vdictdb.h>
#include <vddatabase.h>
#include <vdadddb.h>
#include <vdproperty.h>
#include <vdimport.h>
#include <vdscan.h>
#include <resource.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qframe.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qtooltip.h>
#include <qlayout.h>
#include <qhbox.h>
/*--------------------------------------------------------------------------*/
#include <arrow_up.xpm>
#include <arrow_down.xpm>
#include <edit_dict.xpm>
#include <add_dict.xpm>
#include <del_dict.xpm>
/*--------------------------------------------------------------------------*/
VDDatabase::VDDatabase(): QDialog( 0, 0, true, 0 )
{
	resize( 350, 260 );
	setCaption( "Library" );

	QGridLayout* layout = new QGridLayout( this );
	layout->setSpacing( 8 );
	layout->setMargin( 8 );

	dbList = new QListBox( this );
	layout->addMultiCellWidget( dbList, 0, 4, 0, 3 );

	struct {
		QToolButton **control;
		const char **image;
		const char *tips;
		const char *slot;
		bool status;
	} btn[] = {
		{ &updateBtn, edit_dict_xpm, "Edit options", SLOT( change() ), false },
		{ &addBtn, add_dict_xpm, "Add new", SLOT( add() ), true },
		{ &deleteBtn, del_dict_xpm, "Delete selection", SLOT( remove() ), false },
		{ &upBtn, arrow_up_xpm, "Move up", SLOT( reorder() ), false },
		{ &dnBtn, arrow_down_xpm, "Move down", SLOT( reorder() ), false },
		{ 0, 0 }
	};

	for( int i = 0; btn[i].control; i++ ) {
		QToolButton *b = new QToolButton( this );
		b->setPixmap( QPixmap(btn[i].image) );
		b->setFixedSize( 32, 32 );
		b->setAutoRaise( true );
		b->setEnabled( btn[i].status );
		layout->addWidget( *btn[i].control = b, i, 4 );
		connect( b, SIGNAL( clicked() ), btn[i].slot );
		QToolTip::add( b, btn[i].tips );
	}

	QFrame *f = new QFrame( this );
	f->setFrameStyle( QFrame::HLine | QFrame::Sunken );
	layout->addMultiCellWidget( f, 5, 5, 0, 4 );

	QHBox *b = new QHBox( this );
	layout->addMultiCellWidget( b, 6, 6, 0, 4 );
	importBtn = new QPushButton( "&Import", b );
	searchBtn = new QPushButton( "&Search", b );
	closeBtn = new QPushButton( "&Close", b );
	closeBtn->setDefault( true );
	b->setSpacing( 16 );

	QToolTip::add( importBtn, "Import Webdict\ntext database" );
	QToolTip::add( searchBtn, "Search for existing\nVdict database" );

	connect( importBtn, SIGNAL( clicked() ), SLOT( import() ) );
	connect( searchBtn, SIGNAL( clicked() ), SLOT( search() ) );
	connect( closeBtn, SIGNAL( clicked() ), SLOT( hide() ) );
	connect( dbList, SIGNAL( highlighted(int) ), SLOT( selectDbItem(int) ) );
}
/*--------------------------------------------------------------------------*/
void VDDatabase::execute()
{
	int i, count = dictList.count();

	dbList->clear();
	for( i = 0; i < count; i++ )
		dbList->insertItem(QString::fromUtf8(dictList.at(i)->getDescription()));
	if( count > 0 ) dbList->setCurrentItem( 0 );
	QDialog::exec();
}
/*--------------------------------------------------------------------------*/
void VDDatabase::reorder()
{
	uint n = dbList->currentItem(), np;

	if( sender() == upBtn ) {
		if( n <= 0 ) return;
		np = n - 1;
	}
	else {
		if( n >= dbList->count() - 1 ) return;
		np = n + 1;
	}
	VDictDB *d = dictList.take( n );
	dbList->removeItem( n );
	dbList->insertItem( QString::fromUtf8( d->getDescription() ), np );
	dictList.insert( np, d );
	dbList->setCurrentItem( np );
	upBtn->setEnabled( np > 0 );
	dnBtn->setEnabled( np < dictList.count() - 1 );
	emit changedOrder( n, np );
}
/*--------------------------------------------------------------------------*/
void VDDatabase::add()
{
	VDAddDBDialog addDialog;
	if( addDialog.execute() ) {
		uint n = dictList.count();
		QString dbf = addDialog.getDbfName();
		QString idf = addDialog.getIdfName();
		VDictDB::load( idf, dbf );
		if( n < dictList.count() ) {
			VDictDB *d = dictList.last();
			dbList->insertItem( QString::fromUtf8( d->getDescription() ) );
			dbList->setCurrentItem(dictList.count() - 1);
			emit updated(dictList.count() - 1);
		}
	}
}
/*--------------------------------------------------------------------------*/
void VDDatabase::change()
{
	int p = dbList->currentItem();
	VDictDB *dict = dictList.at( p );
	VDPropertyDialog property( dict );
	if( property.execute() ) {
		dbList->changeItem( QString::fromUtf8(dict->getDescription()), p );
		emit changedInfo( p );
	}
}
/*--------------------------------------------------------------------------*/
void VDDatabase::remove()
{
	int idx = dbList->currentItem();
	
	if( idx >= 0 &&
		QMessageBox::warning( 0, VD_TITLE,
			"Are you sure to remove \""+dbList->currentText()+"\""
			" from resource list ?", "Yes", "No", "", 1, 1 ) == 0 )
	{
		emit deleted( idx );
		dictList.remove( idx );
		dbList->removeItem( idx );
		updateBtn->setEnabled( false );
		deleteBtn->setEnabled( false );
	}
}
/*--------------------------------------------------------------------------*/
void VDDatabase::selectDbItem(int n)
{
	if( n < 0 ) return;
	upBtn->setEnabled( n > 0 );
	dnBtn->setEnabled( n < dictList.count() - 1 );
	updateBtn->setEnabled( true );
	deleteBtn->setEnabled( true );
}
/*--------------------------------------------------------------------------*/
void VDDatabase::import()
{
	QString fn = QFileDialog::getOpenFileName( "",
					"Text files (*.txt *.src);;All files (*)",
					this, 0, "Import Webdict database" ).stripWhiteSpace();
	if( !fn.isEmpty() ) {
		VDImportDialog vdImport( this );
		vdImport.setSource( fn );
		if( vdImport.execute() ) {
			uint n = dictList.count();
			QString dbf = vdImport.getDbfName();
			QString idf = vdImport.getIdfName();
			VDictDB::load( idf, dbf );
			if( n < dictList.count() ) {
				VDictDB *d = dictList.last();
				dbList->insertItem( QString::fromUtf8( d->getDescription() ) );
				dbList->setCurrentItem(dictList.count() - 1);
				emit updated(dictList.count() - 1);
			}
		}
	}
}
/*--------------------------------------------------------------------------*/
void VDDatabase::search()
{
	VDSearchDialog vdSearchDialog( this );
	if( vdSearchDialog.execute() ) {
		int n = dictList.count();
		VDScanner scanner( this, vdSearchDialog.getDirList());
		if( scanner.execute() && n < dictList.count() ) {
			for( int i = n; i< dictList.count(); i++ ) {
				dbList->insertItem(QString::fromUtf8(dictList.at(i)->getDescription()));
				emit updated( i );
			}
		}
	}
}
/*--------------------------------------------------------------------------*/
