/*----------------------------------------------------------------------------*/
/*  vdictdb.cpp                                                               */
/*----------------------------------------------------------------------------*/
/*  copyright         : (C) 2002 by Dao Hai Lam                               */
/*                      VISC Software & Security Consultant Company           */
/*                      Hall 3, Quang Trung Software City                     */
/*                      Tan Chanh Hiep Ward, District 12,                     */
/*                      Ho Chi Minh city, VIETNAM                             */
/*  website           : http://www.visc-network.com                           */
/*  email             : daohailam at yahoo dot com                            */
/*  last modify       : Sat, 27 Sep 2003 14:31:05 +0700                       */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*   This program is free software; you can redistribute it and/or modify     */
/*   it under the terms of the GNU General Public License as published by     */
/*   the Free Software Foundation; either version 2 of the License, or        */
/*   (at your option) any later version.                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <qbuffer.h>
#include <qcstring.h>
#include <qmessagebox.h>
/*----------------------------------------------------------------------------*/
#include "vdconsts.h"
#include "resource.h"
#include "vdictdb.h"
#include "utf.h"
/*----------------------------------------------------------------------------*/
#define VD_IM_LATIN		0
#define VD_IM_VIET		1
/*----------------------------------------------------------------------------*/
QList<VDictDB> dictList;
/*----------------------------------------------------------------------------*/
static char *dfgets(GDBM_FILE fp, char *k)
{
	static char buf[64+4];
	static datum data, key;

	key.dptr = k;
	key.dsize = strlen( k );
	data = gdbm_fetch( fp, key );
	if( !data.dptr )
		return NULL;
	else {
		int n = data.dsize > 64 ? 64 : data.dsize;
		strncpy( buf, data.dptr, n );
		free( data.dptr );
		buf[ n ] = 0;
	}
	return buf;
}
/*----------------------------------------------------------------------------*/
VDictDB::VDictDB( const char *findex, const char *fdata )
{
	mm = NULL;
	ifp = NULL; 
	info.update( fdata, findex );
	dfp = dfopen((char *)fdata, GDBM_READER);
	if( !dfp )
		status = VD_DATA_NOT_FOUND;
	else
		status = loadIndex();
	current = -1;
}
/*----------------------------------------------------------------------------*/
VDictDB::~VDictDB()
{
	if( mm ) munmap( mm, info.size );
	if( ifp ) vfclose( ifp );
	if( dfp ) dfclose( dfp );
}
/*----------------------------------------------------------------------------*/
void VDictDB::saveResource( char* fn )
{
	info.save( fn );
}
/*----------------------------------------------------------------------------*/
long VDictDB::loadIndex()
{
	ifp = vfopen( info.index, "r" );
	if( !ifp )
		return VD_INDEX_NOT_FOUND;

	struct stat fs;
	int fd = fileno( ifp );
	if( fstat( fd, &fs ) < 0 )
		return VD_INDEX_NOT_FOUND;

	int fsize = info.size = fs.st_size;
	header = (VDFMHeader *)(mm = mmap(0, fsize, PROT_READ, MAP_SHARED, fd, 0));

	if( strncmp( header->signature, VD_SIGNATURE, VD_KEYSIZE ) )
		return VD_INDEX_BAD_FORMAT;

	char buf[64], *p;
	char *s = dfgets(dfp, VD_CHECKSUM);
	sprintf(buf, "%lu", header->checksum);
	if( strcmp( buf, s ) )
		return VD_CHECKSUM_ERROR;

	if( !(s = dfgets(dfp, VD_SIGNATURE)) )
		return VD_INDEX_BAD_FORMAT;

	if( strcmp( s, p = ctime(&header->date) ) ) {
		QString msg;
		msg.sprintf("Created dates not match!\nDatabase:  %sIndex:  %s"
					"Do you want to update and continue to use?", s, p);
		if( QMessageBox::warning( 0, VD_TITLE, msg, "&Update", "&Abort", 0, 0, 1 )==1 )
			return VD_DB_NOT_MATCH;
		dfclose(dfp);
		dfp = dfopen(info.data, GDBM_WRITER);
		dfchange(dfp, VD_SIGNATURE, VD_KEYSIZE, p, strlen(p));
		dfclose(dfp);
		dfp = dfopen(info.data, GDBM_READER);
	}

	s = dfgets(dfp, VD_DESCRIPTION);
	info.description = strdup( s ? s : "Noname" );
	
	wordList.data = (char *)((long)mm + sizeof(VDFMHeader));
	wordList.index = (long *)((long)mm + header->offset);
	return VD_READY;
}
/*----------------------------------------------------------------------------*/
void VDictDB::setInfo( QString description, int method )
{
	if( !description.isEmpty() ) {
		if( info.description ) free( info.description );
		char *s = info.description = strdup( description.utf8() );
		dfclose(dfp);
		dfp = dfopen(info.data, GDBM_WRITER);
		dfchange(dfp, VD_DESCRIPTION, VD_KEYSIZE, s, strlen(s));
		dfclose(dfp);
		dfp = dfopen(info.data, GDBM_READER);
	}
	if( method >= 0 && method <= 1 ) {
	}
}
/*----------------------------------------------------------------------------*/
bool VDictDB::getContents( QString word, QTextBrowser *text )
{
	QString s = word.lower();
	long idx = lookup( s );
	if( recent->add( idx, match ) )
		text->setText( getContents( idx ) );
	return true;
}
/*----------------------------------------------------------------------------*/
bool VDictDB::getContents(long i, QTextBrowser *text)
{
	if( recent->add( i, getWord( i ) ) )
		text->setText( getContents( i ) );
	return true;
}
/*----------------------------------------------------------------------------*/
char *VDictDB::getWord( long n )
{
	return match = &wordList.data[ wordList.index[n] ];
}
/*----------------------------------------------------------------------------*/
char *VDictDB::getContents( char *word )
{
	static datum key = { 0, 0 };
	static datum data = { 0, 0 };

	if( word ) {
		key.dptr = word;
		key.dsize = strlen( word );
		data = gdbm_fetch( dfp, key );
	}

	if( !data.dptr )
		return NULL;

	static char line[256];
	static char *p = NULL, *q = NULL;
	char *s = NULL;

	if( !p ) {
		p = data.dptr;
		q = p + data.dsize;
	}

	s = p;
	p = strchr(p, '\n');
	if( p && p<q )
		*p++ = 0;
	else {
		strncpy( line, s, q - s );
		free( data.dptr );
		p = q = data.dptr = NULL;
		s = line;
	}

	return s;
}
/*----------------------------------------------------------------------------*/
#define S_NONE			0
#define S_ITEM			1
#define S_IDIOM			2
#define D_ITEM(a)		"<H3>"VD_ITEM_TAG"<FONT COLOR=\"%s\">%s</FONT></H3>", \
						Resource::defaultColors[1], a
#define D_SPEECH(a)		"<H3>"VD_SPEECH_TAG"<FONT COLOR=\"%s\">[%s]</FONT></H3>", \
						Resource::defaultColors[0], a
#define D_OTHERS(a,b)	"<H3>"VD_OTHERS_TAG"<FONT COLOR=\"%s\">%s</FONT><BR>" \
						VD_SUB_TAG"<FONT COLOR=\"%s\">[%s]</FONT></H3>", \
						Resource::defaultColors[1],a,Resource::defaultColors[0], b
#define D_IDIOMS()		"<H3>"VD_IDIOMS_TAG"<FONT COLOR=\"%s\">Idioms </FONT></H3><OL>", \
						Resource::defaultColors[0]
/*----------------------------------------------------------------------------*/
static char line[4069];
/*----------------------------------------------------------------------------*/
QString VDictDB::highlight(char *word, char *p)
{
	QString s;
	char c=0;

	while( p ) {
		char cc = 0, *sp, *pp;
		
		sp = strchr(p, ' ');
		if( sp ) {
			cc = *(sp-1);
			if( cc=='$' ) *(sp-1) = 0;
			*sp++ = 0;
		}
		if( c=='$' ) {
			while( (pp = strchr(p, '$'))!=NULL )
				strcpy( pp, pp+1 );
			while( (pp = strchr(p, '_'))!=NULL ) *pp = ' ';
			sprintf( line, VD_LINK_TAG"<FONT COLOR=\"%s\">%s</FONT> ", 
						Resource::defaultColors[3], p );
			s += line;
			if( sp && *sp=='$' ) sp++;
		}
		else {
			if( *p=='$' ) p++;
			while( (pp = strchr(p, '_'))!=NULL ) *pp = ' ';
			char z = tolower(*p);
			pp = ((z=='c'||z=='d'||z=='l'||z=='s') && p[1]=='\'') ? p+2 : p;
			if( strcmp(word, pp) )
				sprintf( line, "<A HREF=\"%s\">%s</A> ", pp, p );
			else
				sprintf( line, VD_LINK_TAG"<FONT COLOR=\"%s\">%s</FONT> ",
							Resource::defaultColors[3], p );
			s += line;
		}
		c = cc;
		p = sp;
	}
	return s;
}
/*----------------------------------------------------------------------------*/
QString VDictDB::getContents(long i)
{
	if( i >= 0 && i < header->count ) {
		char *p, *word;
		int session = S_NONE;
		QCString data;

		if( !(word = getWord(i)) ) {
			return QString::null;
		}

		if( !(p = getContents(word)) ) {
			return QString::null;
		}

		data.sprintf(
			"<HTML>"
			"<BODY BGCOLOR=\"#FFFFFF\" LINK=\"%s\" ALINK=\"%s\">",
//			"<FONT COLOR=\"%s\">",
			Resource::defaultColors[3], Resource::defaultColors[3] );
//			Resource::defaultColors[2] );
		if( *p == '/' ) {
			p = strtok(p + 1, "/");
			sprintf( line, D_SPEECH( p ) );
			data += line;
			if( (p = strtok(NULL, ":"))!=NULL ) {
				sprintf( line, "<H3>%s:</H3>", p );
				data += line;
				p = strtok(NULL, "/");
				while( p ) {
					while( *p==' ' || *p=='(' || *p=='$' ) p++;
					*strchr(p, ')') = 0;
					sprintf( line, D_OTHERS( p, strtok(NULL, "/") ) );
					data += line;
					p = strtok(NULL, "/");
				}
			}
		}

		while( true ) {
			char c, *lp = getContents();

			if( !lp ) break;
			c = *lp++;

			switch( c ) {
				case '*':
					if( session==S_IDIOM )
						data += "</UL></OL>";
					else
					if( session==S_ITEM )
						data += "</UL>";
					session = S_ITEM;
					if( (p = strchr(lp, '$'))!=NULL ) *p = '-';
					sprintf( line, D_ITEM( lp ) );
					data += line;
					data += "<UL>";
					break;
				case '-':
					if( session==S_NONE )
						sprintf( line, "- %s<BR> ", lp );
					else {
						if( (p = strchr(lp, '$'))!=NULL ) *p++ = 0;
						if( !p )
							sprintf( line, "<LI>%s<BR> ", lp );
						else {
							char *pp = strchr(p, '$');
							if( !pp ) {
								int z = sprintf( line, "<LI>%s", lp);
								pp = strtok( p, "," );
								while( pp ) {
									z += sprintf( &line[z], "<A HREF=\"%s\">%s</A>,", pp, pp );
									pp = strtok( NULL, "," );
								}
								line[z-1] = 0;
								sprintf( &line[z], "<BR> " );
							}
							else {
								while( pp-1>p && ispunct(*(pp-1)) ) {
									*pp = *(pp-1);
									pp--;
								}
								*pp++ = 0;
								sprintf( line, "<LI>%s<A HREF=\"%s\">%s</A>%s<BR> ", lp, p, p, pp );
							}
						}
					}
					data += line;
					break;
				case '=':
					c = 0;
					while ( *lp == ' ' ) lp++;
					data += highlight( word, strtok( lp, "+" ) ) + "<BR>";
					sprintf( line, "<EM>%s</EM><BR>", strtok( NULL, "\0" ) );
					data += line;
					break;
				case '!':
					data += "</UL>";
					if( session!=S_IDIOM ) {
						session = S_IDIOM;
						sprintf( line, D_IDIOMS() );
						data += line;
					}
					data += "<LI>" + highlight( word, &lp[1] ) + "<UL>";
					break;
			}
		}
		data += "</UL>";
		if( session==S_IDIOM ) data += "</OL>";
		data += "</FONT></BODY></HTML>";
		return QString::fromUtf8( (const char*)data );
	}
	return "<CENTER><H2>Word not found!</H2></CENTER><HR>";
}
/*----------------------------------------------------------------------------*/
long VDictDB::lookup(QString w)
{
	if( !isReady() ) return -1;
	
	long i, c;
	long l = 0, r = header->count-1;
	long latin = header->method==VD_IM_LATIN;

	if( latin )
		w = w.latin1();
	w = w.lower();

	QString s = w.right( 2 );
	if( s == "'s" || s == "'d" )
		w = w.left( w.length() - 2 );
	else {
		s = w.right( 3 );
		if( s == "'ve" || s == "'ll" || s == "'re" || s == "'em" )
			w = w.left( w.length() - 3 );
	}

	if( current >= 0 ) {
		c = ustrcmp( w, QString::fromUtf8( getWord( current ) ), latin );
		if( exact = (c == 0) ) return current;
		if( c < 0 )
			r = current;
		else
			l = current;
	}

	while( true ) {
		if( r-l < 2 ) {
			i = ustrcmp( w, QString::fromUtf8( getWord( l ) ), latin );
			exact = i == 0;
			i = i > 0 ? r : l;
			break;
		}
		if( exact = (c=ustrcmp( w, QString::fromUtf8( getWord(i=(l+r)/2) ), latin ))==0 ) break;
		if( c < 0 )
			r = i;
		else
			l = i;
	}
	return current = i;
}
/*----------------------------------------------------------------------------*/
void VDictDB::load(const char *findex, const char *fdata)
{
	QString msg;
	VDictDB *vdict;

	vdict = new VDictDB( findex, fdata );
	switch( vdict->getStatus() ) {
		case VD_DATA_NOT_FOUND:
			printf("[vdict] gdbm_error(%d) = %s\n", gdbm_errno, gdbm_strerror(gdbm_errno));
			msg.sprintf("Data file %s not found !\n"
						"Please check and try again !", fdata );
			QMessageBox::information( 0, VD_TITLE, msg );
			break;
		case VD_INDEX_NOT_FOUND:
		case VD_INDEX_BAD_FORMAT:
			msg.sprintf( "Index file %s not found or incorrect format", findex );
			QMessageBox::critical( 0, VD_TITLE, msg );
			break;
		case VD_NOT_ENOUGH_MEMORY:
			msg.sprintf("Not enough memory to load %s !", fdata );
			QMessageBox::critical( 0, VD_TITLE, msg );
			break;
		case VD_DB_NOT_MATCH:
			QMessageBox::critical( 0, VD_TITLE, "Database & Index file not match!" );
			break;
		case VD_CHECKSUM_ERROR:
			QMessageBox::critical( 0, VD_TITLE, "Checksum error!\n"
					"Maybe Database & Index file not match!" );
			break;
		default:
			dictList.append(vdict);
			return;
	}
	delete vdict;
}
/*----------------------------------------------------------------------------*/
