/*----------------------------------------------------------------------------*/
/*  vdictdb.h                                                                 */
/*----------------------------------------------------------------------------*/
/*  copyright         : (C) 2002 by Dao Hai Lam                               */
/*                      VISC Software & Security Consultant Company           */
/*                      Hall 3, Quang Trung Software City                     */
/*                      Tan Chanh Hiep Ward, District 12,                     */
/*                      Ho Chi Minh city, VIETNAM                             */
/*  website           : http://www.visc-network.com                           */
/*  email             : daohailam at yahoo dot com                            */
/*  last modify       : Sat, 27 Sep 2003 14:31:05 +0700                       */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*   This program is free software; you can redistribute it and/or modify     */
/*   it under the terms of the GNU General Public License as published by     */
/*   the Free Software Foundation; either version 2 of the License, or        */
/*   (at your option) any later version.                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#ifndef VDICTDB_H
#define VDICTDB_H
/*--------------------------------------------------------------------------*/
#include <time.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
/*--------------------------------------------------------------------------*/
#include <qlist.h>
#include <qlabel.h>
/*--------------------------------------------------------------------------*/
#include <vdict.h>
#include <vddbm.h>
/*--------------------------------------------------------------------------*/
#define VD_HISTORY_SIZE			10
/*--------------------------------------------------------------------------*/
typedef struct {
	long *index;
	char *data;
} VDFMWordList;
/*--------------------------------------------------------------------------*/
typedef struct {
	char signature[8];
	time_t date;
	long method;
	long count;
	long checksum;
	long offset;
} VDFMHeader;
/*----------------------------------------------------------------------------*/
class VDFMFileInfo
{
public:
	char *data;
	char *index;
	char *description;
	long size;

public:
	VDFMFileInfo()
	{
		data = NULL;
		index = NULL;
		description = NULL;
	}

	VDFMFileInfo(const char *fdata, const char *findex)
	{
		update( fdata, findex );
		description = NULL;
	}
	
	~VDFMFileInfo()
	{
		if( data ) free( data );
		if( index ) free( index );
		if( description ) free( description );
	}

	void update(const char *fdata, const char *findex)
	{
		data = strdup( fdata );
		index = strdup( findex );
	}

	void save(const char *fn)
	{
		FILE *fp = fopen( fn, "a+" );
		if( fp ) {
			fprintf( fp, "<VD %s:%s>\n", data, index );
			fclose( fp );
		}
	}
};
/*--------------------------------------------------------------------------*/
class VDictDB : public VDictionary
{
private:
	void *mm;
	FILE *ifp;
	GDBM_FILE dfp;

	char *match;
	long current;
	bool exact;

	VDFMHeader *header;
	VDFMFileInfo info;
	VDFMWordList wordList;
private:
	char *getContents( char *word = NULL );
	long loadIndex();

	QString getContents( long i );
	QString highlight( char *word, char *p );
public:
	VDictDB( const char *findex, const char *fdata );
	~VDictDB();
public:
	char *getWord( long n );
	long getWordCount() const { return header ? header->count : 0; }
	char *getDescription() const { return info.description; }
	char *getDatabaseFile() const { return info.data; }
	char *getIndexFile() const { return info.index; }
	bool isLatin() const { return header ? !header->method : true; }
	bool isExact() const { return exact; }

	long lookup( QString word );
	bool getContents( long i, QTextBrowser *text );
	bool getContents( QString word, QTextBrowser *text );
	void saveResource( char* );

	void setInfo( QString description, int method );

	static void load( const char *findex, const char *fdata );
};
/*----------------------------------------------------------------------------*/
extern QList<VDictDB> dictList;
/*----------------------------------------------------------------------------*/
#endif
