/*----------------------------------------------------------------------------*/
/*  vdictpro.cpp - vdict property widget                                      */
/*----------------------------------------------------------------------------*/
/*  copyright         : (C) 2002 by Dao Hai Lam                               */
/*                      VISC Software & Security Consultant Company           */
/*                      Hall 3, Quang Trung Software City                     */
/*                      Tan Chanh Hiep Ward, District 12,                     */
/*                      Ho Chi Minh city, VIETNAM                             */
/*  website           : http://www.visc-network.com                           */
/*  email             : daohailam at yahoo dot com                            */
/*  last modify       : Sat, 27 Sep 2003 14:31:05 +0700                       */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*   This program is free software; you can redistribute it and/or modify     */
/*   it under the terms of the GNU General Public License as published by     */
/*   the Free Software Foundation; either version 2 of the License, or        */
/*   (at your option) any later version.                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#include <vdictpro.h>
#include <qlabel.h>
#include <qlayout.h>
/*----------------------------------------------------------------------------*/
VDictProperty::VDictProperty( QWidget *parent, VDictDB *dict ): QWidget( parent )
{
	QGridLayout *layout = new QGridLayout( this );
	layout->setSpacing( 8 );

	QLabel *l;
	l = new QLabel( "Description:", this );
	l->setAlignment( Qt::AlignVCenter | Qt::AlignRight );
	layout->addMultiCellWidget( l, 0, 0, 0, 0 );
	description = new VDVietEdit( this );
	layout->addMultiCellWidget( description, 0, 0, 1, 4 );
	
	l = new QLabel( "Lookup word by:", this );
	l->setAlignment( Qt::AlignVCenter | Qt::AlignRight );
	layout->addMultiCellWidget( l, 1, 1, 0, 0 );
	inputMethod = new QComboBox( this );
	layout->addMultiCellWidget( inputMethod, 1, 1, 1, 4 );
	inputMethod->insertItem( "Latin input method" );
	inputMethod->insertItem( "Vietnamese input method" );

	l = new QLabel( "Plural postfix:", this );
	l->setAlignment( Qt::AlignVCenter | Qt::AlignRight );
	layout->addMultiCellWidget( l, 2, 2, 0, 0 );
	pluralForm = new QLineEdit( "-s, -es", this );
	layout->addMultiCellWidget( pluralForm, 2, 2, 1, 4 );
	pluralForm->setEnabled( false );

	autoUpdate = new QCheckBox( "Autoupdate missing word from dictd server", this );
	layout->addMultiCellWidget( autoUpdate, 3, 3, 0, 4 );
	autoUpdate->setEnabled( false );
	QLineEdit *dictServer = new QLineEdit( "<server:port>[/<database>]", this );
	layout->addMultiCellWidget( dictServer, 4, 4, 0, 4 );
	dictServer->setEnabled( false );

	if( dict ) {
		description->setText( QString::fromUtf8(dict->getDescription()) );
		inputMethod->setCurrentItem( dict->isLatin() ? 0 : 1 );
		/*
		pluralForm->setText( dict->getPluralForm() );
		dictServer->setText( dict->getDictServer() );
		autoUpdate->setChecked( dict->isAutoUpdate() );
		*/
	}
	description->setFocus();

	connect(description, SIGNAL( textChanged( const QString& ) ),
			SIGNAL( infoChanged() ));
	connect(inputMethod, SIGNAL( activated( const QString& ) ),
			SIGNAL( infoChanged() ));
	connect(pluralForm, SIGNAL( textChanged( const QString& ) ),
			SIGNAL( infoChanged() ));
	connect(dictServer, SIGNAL( textChanged( const QString& ) ),
			SIGNAL( infoChanged() ));
	this->dict = dict;
}
/*--------------------------------------------------------------------------*/
void VDictProperty::setEnabled( bool e )
{
	description->setEnabled( e );
	inputMethod->setEnabled( e );
	/*
	pluralForm->setEnabled( e );
	dictServer->setEnabled( e );
	autoUpdate->setEnabled( e );
	*/
	QWidget::setEnabled( e );
}
/*--------------------------------------------------------------------------*/
void VDictProperty::update()
{
	if( dict )
		dict->setInfo( description->text(), inputMethod->currentItem() );
}
/*--------------------------------------------------------------------------*/
