/*----------------------------------------------------------------------------*/
/*  vdimport.cpp                                                              */
/*----------------------------------------------------------------------------*/
/*  copyright         : (C) 2002 by Dao Hai Lam                               */
/*                      VISC Software & Security Consultant Company           */
/*                      Hall 3, Quang Trung Software City                     */
/*                      Tan Chanh Hiep Ward, District 12,                     */
/*                      Ho Chi Minh city, VIETNAM                             */
/*  website           : http://www.visc-network.com                           */
/*  email             : daohailam at yahoo dot com                            */
/*  last modify       : Sat, 27 Sep 2003 14:31:05 +0700                       */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*   This program is free software; you can redistribute it and/or modify     */
/*   it under the terms of the GNU General Public License as published by     */
/*   the Free Software Foundation; either version 2 of the License, or        */
/*   (at your option) any later version.                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#include <stdarg.h>
/*----------------------------------------------------------------------------*/
#include <vdconsts.h>
#include <vdictdb.h>
#include <vdimport.h>
#include <vdwidgets.h>
#include <resource.h>
#include <qapplication.h>
#include <qfiledialog.h>
#include <qframe.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qmessagebox.h>
#include <qlayout.h>
#include <qhbox.h>
/*----------------------------------------------------------------------------*/
#include <browse.xpm>
/*----------------------------------------------------------------------------*/
VDImportDialog *importer = NULL;
/*----------------------------------------------------------------------------*/
int create_vdbf(const char *sfn, const char *dfn, const char *ifn, long method,
				const char *description);
/*----------------------------------------------------------------------------*/
void set_word_counter( int n )
{
	QString c;
	c.setNum( n );
	importer->counter->setText( c );
	qApp->processEvents();
}
/*----------------------------------------------------------------------------*/
void show_error_message(char *fmt, ...)
{
	va_list ap;
	char buf[256];
	va_start(ap, fmt);
	vsnprintf(buf, sizeof(buf), fmt, ap);
	va_end(ap);

	QMessageBox::critical( importer, "Error", buf );
}
/*--------------------------------------------------------------------------*/
VDImportDialog::VDImportDialog( QWidget *parent ): QDialog( parent, 0, true, 0 )
{
	resize( 360, 150 );
	setCaption( "Import Webdict database" );

	QGridLayout *layout = new QGridLayout( this );
	layout->setSpacing( 8 );
	layout->setMargin( 8 );

	QLabel *l;
	l = new QLabel( "Input Webdict database (source):", this );
	layout->addMultiCellWidget( l, 0, 0, 0, 4 );
	srcFile = new QLabel( this );
	layout->addMultiCellWidget( srcFile, 1, 1, 0, 4 );
	srcFile->setFrameShape( QFrame::Box );
	srcFile->setFrameShadow( QFrame::Raised );

	l = new QLabel( "Output Vdict database:", this );
	layout->addMultiCellWidget( l, 2, 2, 0, 4 );
	dbfFile = new QLineEdit( this );
	layout->addMultiCellWidget( dbfFile, 3, 3, 0, 3 );
	dbfBtn = new QToolButton( this );
	dbfBtn->setPixmap( QPixmap(browse_xpm) );
	dbfBtn->setFixedWidth( 32 );
	layout->addWidget( dbfBtn, 3, 4 );

	l = new QLabel( "Output Vdict index:", this );
	layout->addMultiCellWidget( l, 4, 4, 0, 4 );
	idfFile = new QLineEdit( this );
	layout->addMultiCellWidget( idfFile, 5, 5, 0, 3 );
	idfBtn = new QToolButton( this );
	idfBtn->setPixmap( QPixmap(browse_xpm) );
	idfBtn->setFixedWidth( 32 );
	layout->addWidget( idfBtn, 5, 4 );

	dictProperty = new VDictProperty( this );
	layout->addMultiCellWidget( dictProperty, 6, 6, 0, 4 );

	l = new QLabel( "Word count:", this );
	layout->addMultiCellWidget( l, 7, 7, 0, 0 );
	counter = new QLabel( "Press \"Start\" button to begin", this );
	layout->addMultiCellWidget( counter, 7, 7, 1, 4 );
	counter->setFrameShape( QFrame::Box );
	counter->setFrameShadow( QFrame::Raised );

	QFrame *f = new QFrame( this );
	f->setFrameStyle( QFrame::HLine | QFrame::Sunken );
	layout->addMultiCellWidget( f, 8, 8, 0, 4 );

	QHBox *b = new QHBox( this );
	layout->addMultiCellWidget( b, 9, 9, 0, 4 );
	startBtn = new QPushButton( "&Start", b );
	cancelBtn = new QPushButton( "&Cancel", b );
	startBtn->setEnabled( false );
	b->setSpacing( 12 );

	connect( startBtn, SIGNAL( clicked() ), SLOT( start() ) );
	connect( cancelBtn, SIGNAL( clicked() ), SLOT( reject() ) );
	connect( dbfBtn, SIGNAL( pressed() ), SLOT( browse() ) );
	connect( idfBtn, SIGNAL( pressed() ), SLOT( browse() ) );
	connect( dictProperty, SIGNAL( infoChanged() ), SLOT( allow() ) );
	cancelBtn->setDefault( true );
}
/*--------------------------------------------------------------------------*/
void VDImportDialog::setSource(QString wdFile)
{
	int p = wdFile.findRev('/');
	QString db = p >= 0 ? wdFile.right(wdFile.length() - p - 1) : wdFile;
	if( (p = db.findRev('.')) >= 0 ) db = db.left( p );

	srcFile->setText( wdFile );
	dbfFile->setText( Resource::getDataDir() + "/" + db + VDBF_EXT );
	idfFile->setText( Resource::getDataDir() + "/" + db + VDBI_EXT );
}
/*--------------------------------------------------------------------------*/
bool VDImportDialog::execute()
{
	QDialog::exec();
	return result() == QDialog::Accepted;
}
/*--------------------------------------------------------------------------*/
void VDImportDialog::allow()
{
	startBtn->setEnabled( dictProperty->getDescription() != "" );
}
/*--------------------------------------------------------------------------*/
void VDImportDialog::browse()
{
	QToolButton *btn = (QToolButton *)sender();
	QLineEdit *editor = NULL;
	char *caption = NULL;
	char *filter = NULL;

	static char *filters[] = {
		"Vdict data file (*"VDBF_EXT")",
		"Vdict index file (*"VDBI_EXT")"
	};

	static char *captions[] = {
		"Save Vdict data file to",
		"Save Vdict index file to"
	};
	
	if( btn == dbfBtn ) {
		editor = dbfFile;
		caption = captions[0];
		filter = filters[0];
	}
	else
	if( btn == idfBtn ) {
		editor = idfFile;
		caption = captions[1];
		filter = filters[1];
	}

	if( editor ) {
		QString fn = QFileDialog::getSaveFileName( Resource::getDataDir(),
						filter, this, 0, caption ).stripWhiteSpace();
		if( !fn.isEmpty() ) {
			editor->setText( fn );
			int p = fn.findRev('.');
			if( p >= 0 ) fn.truncate( p );
			fn += btn == dbfBtn ? VDBI_EXT : VDBF_EXT;
			if( QFile(fn).exists() ) {
				editor = btn == dbfBtn ? idfFile : dbfFile;
				if( editor->text().stripWhiteSpace() == "" )
					editor->setText( fn );
			}
		}
		btn->setDown(false);
		setFocus();
	}
}
/*--------------------------------------------------------------------------*/
void VDImportDialog::start()
{
	importer = this;
	dbfFile->setEnabled( false ); dbfBtn->setEnabled( false );
	idfFile->setEnabled( false ); idfBtn->setEnabled( false );
	dictProperty->setEnabled( false );
	cancelBtn->setEnabled( false );
	startBtn->setEnabled( false );
	if( create_vdbf(srcFile->text(), dbfFile->text(), idfFile->text(),
			dictProperty->getInputMethod(), dictProperty->getDescription() ) )
	{
		importer = NULL;
		accept();
		return;
	}
	dbfFile->setEnabled( true ); dbfBtn->setEnabled( true );
	idfFile->setEnabled( true ); idfBtn->setEnabled( true );
	dictProperty->setEnabled( true );
	cancelBtn->setEnabled( true );
	startBtn->setEnabled( true );
}
/*--------------------------------------------------------------------------*/

