/*----------------------------------------------------------------------------*/
/*  vdscan.cpp                                                                */
/*----------------------------------------------------------------------------*/
/*  copyright         : (C) 2002 by Dao Hai Lam                               */
/*                      VISC Software & Security Consultant Company           */
/*                      Hall 3, Quang Trung Software City                     */
/*                      Tan Chanh Hiep Ward, District 12,                     */
/*                      Ho Chi Minh city, VIETNAM                             */
/*  website           : http://www.visc-network.com                           */
/*  email             : daohailam at yahoo dot com                            */
/*  last modify       : Sat, 27 Sep 2003 14:31:05 +0700                       */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*   This program is free software; you can redistribute it and/or modify     */
/*   it under the terms of the GNU General Public License as published by     */
/*   the Free Software Foundation; either version 2 of the License, or        */
/*   (at your option) any later version.                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#include <qframe.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qtooltip.h>
#include <qlayout.h>
#include <qhbox.h>
#include <qdir.h>
#include <qtimer.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qapplication.h>
/*--------------------------------------------------------------------------*/
#include <vdconsts.h>
#include <vdictdb.h>
#include <vdscan.h>
#include <resource.h>
/*--------------------------------------------------------------------------*/
#include <add_dict.xpm>
#include <del_dict.xpm>
/*--------------------------------------------------------------------------*/
char *searchList[] = {
	"/usr/share/dict",
	"/usr/local/share/dict",
	"/opt/share",
	0
};
/*--------------------------------------------------------------------------*/
VDSearchDialog::VDSearchDialog( QWidget *parent ): QDialog( parent, 0, true, 0 )
{
	resize( 320, 200 );
	setCaption( "Search for Vdict database" );

	QGridLayout* layout = new QGridLayout( this );
	layout->setSpacing( 8 );
	layout->setMargin( 8 );

	QLabel *l = new QLabel("Search list:", this);
	l->setFixedHeight( parent->font().pointSize() + 4 );
	layout->addMultiCellWidget( l, 0, 0, 0, 4 );

	dirList = new QListBox( this );
	dirList->insertItem( Resource::getDataDir() );
	for( int i = 0; searchList[i]; i++ )
		dirList->insertItem( searchList[i] );
	layout->addMultiCellWidget( dirList, 1, 4, 0, 3 );

	struct {
		QToolButton **control;
		const char **image;
		const char *tips;
		const char *slot;
		bool status;
	} btn[] = {
		{ &addBtn, add_dict_xpm,
			"Add directory\nto search list", SLOT( add() ), true },
		{ &delBtn, del_dict_xpm,
			"Remove directory\n from search list", SLOT( remove() ), false },
		{ 0, 0 }
	};

	for( int i = 0; btn[i].control; i++ ) {
		QToolButton *b = new QToolButton( this );
		b->setPixmap( QPixmap(btn[i].image) );
		b->setFixedSize( 32, 32 );
		b->setAutoRaise( true );
		b->setEnabled( btn[i].status );
		layout->addWidget( *btn[i].control = b, i + 1, 4 );
		connect( b, SIGNAL( clicked() ), btn[i].slot );
		QToolTip::add( b, btn[i].tips );
	}

	QFrame *f = new QFrame( this );
	f->setFrameStyle( QFrame::HLine | QFrame::Sunken );
	layout->addMultiCellWidget( f, 5, 5, 0, 4 );

	QHBox *b = new QHBox( this );
	layout->addMultiCellWidget( b, 6, 6, 0, 4 );
	startBtn = new QPushButton( "&Start", b );
	closeBtn = new QPushButton( "&Close", b );
	closeBtn->setDefault( true );
	b->setSpacing( 16 );

	connect( startBtn, SIGNAL( clicked() ), SLOT( accept() ) );
	connect( closeBtn, SIGNAL( clicked() ), SLOT( reject() ) );
	connect( dirList, SIGNAL( highlighted(int) ), SLOT( selectItem(int) ) );
}
/*--------------------------------------------------------------------------*/
bool VDSearchDialog::execute()
{
	QDialog::exec();
	return result() == QDialog::Accepted;
}
/*--------------------------------------------------------------------------*/
void VDSearchDialog::add()
{
	QString d = QFileDialog::getExistingDirectory( "", this,
					"get existing directory", "Choose a directory",	TRUE );
	if( !d.isEmpty() ) dirList->insertItem( d );
	startBtn->setEnabled( true );
}
/*--------------------------------------------------------------------------*/
void VDSearchDialog::remove()
{
	int idx = dirList->currentItem();
	if( idx >= 0 ) {
		dirList->removeItem( idx );
		delBtn->setEnabled( false );
	}
	if( !dirList->count() )
		startBtn->setEnabled( false );
}
/*--------------------------------------------------------------------------*/
void VDSearchDialog::selectItem(int n)
{
	delBtn->setEnabled( n >= 0 );
}
/*--------------------------------------------------------------------------*/
QStringList VDSearchDialog::getDirList()
{
	QStringList d;
	for( int i = 0; i < dirList->count(); i++ )
		d.append( dirList->text( i ) );
	return d;
}
/*--------------------------------------------------------------------------*/
/* VDScanber class                                                          */
/*--------------------------------------------------------------------------*/
VDScanner::VDScanner( QWidget *parent ): QDialog( parent, 0, true )
{
	dirs.append( Resource::getDataDir() );
	for( int i = 0; searchList[i]; i++ )
		dirs.append( searchList[i] );
	init();
}
/*--------------------------------------------------------------------------*/
VDScanner::VDScanner( QWidget *parent, QStringList dirs ): QDialog( parent, 0, true )
{
	this->dirs = dirs;
	init();
}
/*--------------------------------------------------------------------------*/
void VDScanner::init()
{
	resize( 360, 120 );
	setCaption( "Searching for Vdict database" );

	QGridLayout* layout = new QGridLayout( this );
	layout->setSpacing( 8 );
	layout->setMargin( 8 );

	QLabel *l = new QLabel( "Searching in:", this );
	l->setAlignment( Qt::AlignVCenter | Qt::AlignRight );
	l->setFixedWidth( 120 );
	layout->addWidget( l, 0, 0 );
	l = new QLabel( "Add:", this );
	l->setAlignment( Qt::AlignVCenter | Qt::AlignRight );
	layout->addWidget( l, 1, 0 );
	l = new QLabel( "Total:", this );
	l->setAlignment( Qt::AlignVCenter | Qt::AlignRight );
	layout->addWidget( l, 2, 0 );

	searchDir = new QLabel( this );
	layout->addWidget( searchDir, 0, 1 );
	found = new QLabel( this );
	layout->addWidget( found, 1, 1 );
	counter = new QLabel( "0", this );
	layout->addWidget( counter, 2, 1 );

	QFrame *f = new QFrame( this );
	f->setFrameStyle( QFrame::HLine | QFrame::Sunken );
	layout->addMultiCellWidget( f, 3, 3, 0, 1 );

	QHBox *b = new QHBox( this );
	layout->addMultiCellWidget( b, 4, 4, 0, 1 );
	new QLabel( b );
	QPushButton *btn = new QPushButton( "&Stop", b );
	new QLabel( b );
	btn->setDefault( true );

	connect( btn, SIGNAL( clicked() ), SLOT( stop() ) );
	count = 0;
}
/*--------------------------------------------------------------------------*/
void VDScanner::process()
{
	for( QStringList::Iterator d = dirs.begin(); allow && d != dirs.end(); d++ ) {
		setSearchDir( (const char *)*d );
		qApp->processEvents();

		QStringList l = QDir( *d ).entryList("*"VDBF_EXT);
		for( QStringList::Iterator f = l.begin(); allow && f != l.end(); f++ ) {
			QString dbf = *d + "/" + *f;
			QString idf = dbf.left( dbf.length() - 5 ) + VDBI_EXT;

			int i;
			for( i = 0; i < dictList.count(); i++ )
				if( dbf == dictList.at(i)->getDatabaseFile() ) break;
			if( i >= dictList.count() ) {
				VDictDB *dict = new VDictDB( idf , dbf );
				if( !dict->isReady() )
					delete dict;
				else {
					setFound( dict->getDescription() );
					dictList.append( dict );
					qApp->processEvents();
					sleep( 1 );
				}
			}
		}
	}
	done();
}
/*--------------------------------------------------------------------------*/
bool VDScanner::execute()
{
	allow = true;
	QTimer::singleShot( 200, this, SLOT( process() ) );

	QDialog::exec();
	return result() == QDialog::Accepted;
}
/*--------------------------------------------------------------------------*/
void VDScanner::stop()
{
	if( QMessageBox::warning( this, "Warning",
			"Do you really want to stop?", "&Continue", "&Stop", 0 ) == 1 ) {
		allow = false;
		reject();
	}
}
/*--------------------------------------------------------------------------*/
void VDScanner::done()
{
	QString msg;
	msg.sprintf("Found total %d dictionar%s", count, count > 1 ? "ies" : "y");
	QMessageBox::information( this, VD_TITLE, msg, QMessageBox::Ok );
	accept();
}
/*--------------------------------------------------------------------------*/
void VDScanner::setSearchDir( const char *name )
{
	searchDir->setText( name );
}
/*--------------------------------------------------------------------------*/
void VDScanner::setFound( const char *name )
{
	QString c;
	c.setNum( ++count );
	counter->setText( c );
	found->setText( QString::fromUtf8( name ) );
}
/*--------------------------------------------------------------------------*/
