/*----------------------------------------------------------------------------*/
/*  vdwidgets.cpp                                                             */
/*----------------------------------------------------------------------------*/
/*  copyright         : (C) 2002 by Dao Hai Lam                               */
/*                      VISC Software & Security Consultant Company           */
/*                      Hall 3, Quang Trung Software City                     */
/*                      Tan Chanh Hiep Ward, District 12,                     */
/*                      Ho Chi Minh city, VIETNAM                             */
/*  website           : http://www.visc-network.com                           */
/*  email             : daohailam at yahoo dot com                            */
/*  last modify       : Sat, 27 Sep 2003 14:31:05 +0700                       */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*   This program is free software; you can redistribute it and/or modify     */
/*   it under the terms of the GNU General Public License as published by     */
/*   the Free Software Foundation; either version 2 of the License, or        */
/*   (at your option) any later version.                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#include "visckey.h"
#include "vdconsts.h"
#include "vdwidgets.h"
#include "resource.h"
#include <qkeycode.h>
#include <ctype.h>
/*----------------------------------------------------------------------------*/
VDTextBrowser *contents = NULL;
/*----------------------------------------------------------------------------*/
/* VDVietEdit class                                                           */
/*----------------------------------------------------------------------------*/
VDVietEdit::VDVietEdit(QWidget *parent, const char *name): QLineEdit(parent, name)
{
	latin = false;
}
/*----------------------------------------------------------------------------*/
void VDVietEdit::keyPressEvent(QKeyEvent *event)
{
	int p, l, v=0, code = event->key();
	char key, emitKeyPressed = 1;

	if( !code ) {
		QLineEdit::keyPressEvent( event );
		emit keyPressed();
		return;
	}
	if( latin ) {
		v = visckey;
		visckey = VK_LATIN;
	}
	else
	if( visckey==VK_OFF ) {
		if( code==Key_Escape ) selectAll();
		QLineEdit::keyPressEvent( event );
		emit keyPressed();
		return;
	}
	if( event->state() & (ControlButton|AltButton) )
		VKClearBuffer();
	else
	switch( code ) {
		case Key_Up:
		case Key_Down:
		case Key_PageUp:
		case Key_PageDown:
			VKClearBuffer();
			emit browse( code );
			code = 0;
			break;
		case Key_Escape:
			selectAll();
		case Key_Enter:
		case Key_Return:
		case Key_Home:
		case Key_Left:
		case Key_Right:
		case Key_End:
		case Key_Tab:
			VKClearBuffer();
			emitKeyPressed = 0;
			break;
		case Key_Backspace:
			VKBackDelete();
			break;
		default:
			if( code>0xFF ) {
				emitKeyPressed = 0;
				break;
			}
			if( (l=currentword.length)>30 || (key=event->ascii())==' ' ) {
				VKClearBuffer();
				break;
			}
			if( (p=VKAddKey( key ))>=0 ) {
				int c = cursorPosition();
				setCursorPosition( c-(l-p-1) );
				backspace();
				if( latin )
					insert( QChar( (char)currentword.data[ p ] ) );
				else
					insert( QChar( currentword.data[ p ] ) );
				setCursorPosition( c );
				if( currentword.length==l ) code = 0;
			}
	}
	if( code )
		QLineEdit::keyPressEvent( event );
	if( emitKeyPressed ) {
		emit keyPressed();
	}
	if( latin ) visckey = v;
};
/*----------------------------------------------------------------------------*/
void VDVietEdit::focusOutEvent( QFocusEvent *fe )
{
	VKClearBuffer();
	QLineEdit::focusOutEvent( fe );
}
/*----------------------------------------------------------------------------*/
void VDVietEdit::mousePressEvent( QMouseEvent *me )
{
	VKClearBuffer();
	QLineEdit::mousePressEvent( me );
}
/*----------------------------------------------------------------------------*/
// VDTextBrowser class
/*----------------------------------------------------------------------------*/
void VDTextBrowser::setTextColor( const QColor& c )
{
	/*
	QColorGroup cg = paperColorGroup();
	cg.setColor( QColorGroup::Text, c );
	setPaperColorGroup( cg );
	*/
}
/*----------------------------------------------------------------------------*/
void VDTextBrowser::applyDefaultColor()
{
	static struct {
		char *tag;
		char *color;
	} items[] = {
		{ VD_NORMAL_TAG, Resource::defaultColors[2] },
		{ VD_LINK_TAG,   Resource::defaultColors[3] },
		{ VD_ITEM_TAG,   Resource::defaultColors[1] },
		{ VD_SPEECH_TAG, Resource::defaultColors[0] },
		{ VD_OTHERS_TAG, Resource::defaultColors[1] },
		{ VD_SUB_TAG,    Resource::defaultColors[0] },
		{ VD_IDIOMS_TAG, Resource::defaultColors[0] }
	};
	QString s = text();
	
	for( int i, p=i=0; i<5; i++, p=0 )
	while( (p=s.find( items[i].tag, p ))>=0 ) {
		s.replace( p+VD_TAG_LENGTH, 7, items[i].color );
		p++;
	}
	setText( s );
}
/*----------------------------------------------------------------------------*/
// VDMultiLineEdit class
/*----------------------------------------------------------------------------*/
void VDMultiLineEdit::keyPressEvent( QKeyEvent* e )
{
	int p, l, code = e->key();
	char key;

	if( e->state() & (ControlButton|AltButton) ) {
		VKClearBuffer();
		if( code==Key_Insert && (e->state() & ControlButton) )
			copy();
		else
		if( code==Key_Shift && (e->state() & AltButton) ) {
			visckey = (visckey+1) % 3;
			emit methodChanged();
		}
	}
	else
	if( visckey )
	switch( code ) {
		case Key_Insert:
			setOverwriteMode( !isOverwriteMode() );
			emit modeChanged();
		case Key_Up:
		case Key_Down:
		case Key_Left:
		case Key_Right:
		case Key_PageUp:
		case Key_PageDown:
		case Key_Home:
		case Key_End:
		case Key_Tab:
		case Key_Enter:
		case Key_Return:
		case Key_Escape:
			VKClearBuffer();
			break;
		case Key_Backspace:
			VKBackDelete();
			break;
		default:
			if( code>0xFF ) break;
			if( (l=currentword.length)>30 || (key=e->ascii())==' ' ) {
				VKClearBuffer();
				break;
			}
			if( (p=VKAddKey( key ))>=0 ) {
				int r, c;
				getCursorPosition( &r, &c );
				if( isOverwriteMode() )
					setCursorPosition( r, c-(l-p) );
				else {
					setCursorPosition( r, c-(l-p-1) );
					backspace();
				}
				insert( QChar( currentword.data[ p ] ) );
				setCursorPosition( r, c );
				if( currentword.length==l ) code = 0;
			}
	}
	if( code ) QMultiLineEdit::keyPressEvent( e );
	emit keyPressed();
}
/*----------------------------------------------------------------------------*/
void VDMultiLineEdit::focusOutEvent( QFocusEvent* e )
{
	VKClearBuffer();
	QMultiLineEdit::focusOutEvent( e );
}
/*----------------------------------------------------------------------------*/
void VDMultiLineEdit::mousePressEvent( QMouseEvent* e )
{
	VKClearBuffer();
	QMultiLineEdit::mousePressEvent( e );
	emit keyPressed();
}
/*----------------------------------------------------------------------------*/
