#include <time.h>
#include <ctype.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
/*----------------------------------------------------------------------------*/
#include <vddbm.h>
/*----------------------------------------------------------------------------*/
char *methods[] = { "Latin input method", "Vietnamese input method" };
/*----------------------------------------------------------------------------*/
#ifdef __VDICT_GUI__
extern void set_word_counter(int n);
extern void show_error_message(char *fmt, ...);
#else
#define set_word_counter(n)	printf( "Word counts = %d\r", n )
#define show_error_message	printf
void usage(char *prog)
{
	printf( "\n* Usage:\n"
			"    %s [options]\n"
			"\t      -s <webdict.db>   Input HND's WebDict database\n"
			"\t      -d <vd"VDBF_EXT">      Output VISC Dictionary database\n"
			"\t      -i <vd"VDBI_EXT">      Output VISC Dictionary index file\n"
			"\t      -m <vn|lat>       Using Vietnamese/Latin input method (default Latin)\n"
			"\t      -x <description>  Dictionary's description (default input)\n\n",
			prog);
	exit( -1 );
}
#endif
/*----------------------------------------------------------------------------*/
#define stop_import(args...)	{	\
	show_error_message(args);		\
	return 0;						\
}
/*----------------------------------------------------------------------------*/
int create_vdbf(const char *sfn, const char *dfn, const char *ifn, long method,
				const char *description)
{
	time_t date = time(0);

	FILE *ofp = tmpfile();
	FILE *sfp = vfopen( sfn, "r" );
	FILE *ifp = vfopen( ifn, "w+" );
	GDBM_FILE dfp;

	static char buffer[65536];
	char line[1024], word[64];
	char *p, *q, *w = word;
	long offset, count = 0, idx = 0;
	unsigned long checksum = 0;

	if( !sfp ) stop_import( "Cannot open source file: %s!\n", sfn );
	if( !ifp ) stop_import( "Create index[%s] error!\n", ifn );
	if( (dfp = dfopen( (char *)dfn, GDBM_WRCREAT )) == NULL )
		stop_import( "Create db[%s] error: %d\n", dfn, gdbm_errno );

	vfwrite( ifp, VD_SIGNATURE, VD_KEYSIZE );
	vfwrite( ifp, &date, sizeof( date ) );
	vfwrite( ifp, &method, sizeof( method ) );
	vfwrite( ifp, &count, sizeof( count ) );
	vfwrite( ifp, &checksum, sizeof( checksum ) );
	vfwrite( ifp, &idx, sizeof( idx ) );

	*w = 0;
	*(q = buffer) = 0;
	count = checksum = offset = 0;
	while( vfgets( sfp, p=line, sizeof( line ) ) ) {
		long l = strlen( p );
		if( *p != '@' ) {
			strcat( q, p );
			q += l;
		}
		else {
			if( q != buffer ) {
				*q = 0;
				if( word[0] ) {
					if( !strncmp(word, "00-database-", 12) ) {
						static char *stype = "|info|url|short|";
						switch( strstr(stype, &word[12]) - stype ) {
							case 1:
								dfinsert( dfp, VD_DBFINFO, 0, buffer, q - buffer );
								break;
							case 6:
								dfinsert( dfp, VD_DBFURL, 0, buffer, q - buffer );
								break;
							case 10:
								dfinsert( dfp, VD_DBFNAME, 0, buffer, q - buffer );
								break;
						}
					}
					else {
						int c = strlen( word ) + 1;
						dfinsert( dfp, word, 0, buffer, q - buffer );
						vfwrite( ofp, &offset, sizeof( offset ) );
						vfwrite( ifp, word, c );
						offset += c;
					}
				}
				*(q = buffer) = 0;
			}

			set_word_counter( ++count );

			w = p + 1;
			if( (p = strstr( w, " /" ))!=NULL ) {
				strcpy( q, p + 1 );
				q += strlen( q );
			}
			else {
				p = &w[l - 1];
				*q++ = '\n';
				*q = 0;
			}
			while( *p == 0 || isspace(*p) ) p--;
			*(p + 1) = 0;

			while( isspace( *w ) ) w++;
			strcpy( word, w );

			while( *w ) checksum += *(unsigned char *)w++;
		}

		if( q - buffer > 60000 ) {
			vfclose( sfp );
			vfclose( ofp );
			vfclose( ifp );
			dfclose( dfp );
			stop_import( "It seems source file is not HND's WebDict database!\n" );
		}
	}
	vfclose( sfp );

	if( q != buffer ) {
		dfinsert( dfp, word, 0, buffer, q - buffer );
		vfwrite( ofp, &offset, sizeof( offset ) );
		vfwrite( ifp, word, strlen( word ) + 1 );
	}
	idx = vftell( ifp );

	// Append all index table to index file
	vfseek( ofp, 0, SEEK_SET );
	while( !vfeof( ofp ) ) {
		int n = vfread( ofp, buffer, sizeof(buffer) );
		vfwrite( ifp, buffer, n );
	}
	vfclose( ofp );

	// Update all keys to db
	sprintf( buffer, "%lu", checksum );
	dfinsert( dfp, VD_SIGNATURE, VD_KEYSIZE, ctime(&date), 0 );
	dfinsert( dfp, VD_DESCRIPTION, VD_KEYSIZE, (char *)description, 0 );
	dfinsert( dfp, VD_INPUTMETHOD, VD_KEYSIZE, methods[method], 0 );
	dfinsert( dfp, VD_CHECKSUM, VD_KEYSIZE, buffer, 0 );
	dfclose( dfp );

	// Update all other info to index file
	vfseek( ifp, VD_KEYSIZE + sizeof( date ) + sizeof( method ), SEEK_SET );
	vfwrite( ifp, &count, sizeof( count ) );
	vfwrite( ifp, &checksum, sizeof( checksum ) );
	vfwrite( ifp, &idx, sizeof( idx ) );
	vfclose( ifp );
	return 1;
}
/*----------------------------------------------------------------------------*/
#ifndef __VDICT_GUI__
char *dup_fname(char *s, char *ext)
{
	char *p = strrchr(s, '/');
	char *f;
	
	if( p ) s = p + 1;
	f = (char *)malloc( strlen(p) + strlen(ext) );
	strcpy( f, s );
	if( !(p = strrchr(f, '.')) )
		strcat( f, ext );
	else
		strcpy( p, ext );
	return f;
}
/*----------------------------------------------------------------------------*/
main(int argc, char **argv)
{
	long method = 0;
	char *prog = argv[0];
	char *sfn = NULL, *dfn = NULL, *ifn = NULL;
	char *description = NULL;
	
	while( 1 ) {
		int c = getopt( argc, argv, "s:d:i:m:x:" );
		if( c == -1 ) break;
		switch( c ) {
			case 's':
				sfn = optarg;
				break;
			case 'd':
				dfn = optarg;
				break;
			case 'i':
				ifn = optarg;
				break;
			case 'm':
				method = !strcmp(optarg, "vn") ? 1 : 0;
				break;
			case 'x':
				description = optarg;
				break;
		}
	}
	
	if( !sfn ) usage(prog);
	if( !dfn ) dfn = dup_fname(sfn, VDBF_EXT);
	if( !ifn ) ifn = dup_fname(sfn, VDBI_EXT);
	if( !description ) {
		description = (char *)malloc(24+4);
		printf("Description (24 chars): ");
		fgets(description, 26, stdin);
		description[ strlen( description ) - 1 ] = 0;
	}
	
	printf(
		"wddb = %s\nvdbf = %s\nvdbi = %s\nimed = %s\n",
		sfn, dfn, ifn, methods[method]
	);
	create_vdbf( sfn, dfn, ifn, method, description );
	printf("\ndone.\n");
}
#endif
