/*----------------------------------------------------------------------------*/
/*  vd.cpp                                                                    */
/*----------------------------------------------------------------------------*/
/*  copyright         : (C) 2002 by Dao Hai Lam                               */
/*                      VISC Software & Security Consultant Company           */
/*                      Hall 3, Quang Trung Software City                     */
/*                      Tan Chanh Hiep Ward, District 12,                     */
/*                      Ho Chi Minh city, VIETNAM                             */
/*  website           : http://www.visc-network.com                           */
/*  email             : daohailam at yahoo dot com                            */
/*  last modify       : Sat, 27 Sep 2003 14:31:05 +0700                       */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*   This program is free software; you can redistribute it and/or modify     */
/*   it under the terms of the GNU General Public License as published by     */
/*   the Free Software Foundation; either version 2 of the License, or        */
/*   (at your option) any later version.                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#include <unistd.h>
#include <vdconsts.h>
#include <vdictdb.h>
#include <vdcontentviewer.h>
#include <vdeditor.h>
/*----------------------------------------------------------------------------*/
#include "vd.h"
#include "vd.xpm"
/*----------------------------------------------------------------------------*/
#include "resource.h"
#include "vdtoolbar.h"
#include "vdlookupframe.h"
#include "vdlocallookupframe.h"
#include "vdremotelookupframe.h"
#include "vdpreferences.h"
#include "vddatabase.h"
#include "visckey.h"
#include "data.h"
/*----------------------------------------------------------------------------*/
#include <qtooltip.h>
#include <qobjectlist.h>
/*----------------------------------------------------------------------------*/
#define VD_FRAME_SHADOW			QFrame::Raised
#define VD_FRAME_STYLE			QFrame::WinPanel|QFrame::Raised
#define VD_STATUS_BAR_STYLE		QFrame::Panel|QFrame::Sunken
/*----------------------------------------------------------------------------*/
VDEditor *editor;
/*----------------------------------------------------------------------------*/
VD::VD() : QWidget( 0, 0, WDestructiveClose )
{
	setCaption( VD_TITLE );
	setIcon( QPixmap( (const char**)vd_xpm ) );
	Resource::load("vdrc");

	QBoxLayout *topLayout = new QVBoxLayout( this, 2 );
	
	VDToolBar *toolbar = new VDToolBar( this );
	connect( toolbar, SIGNAL( configure() ), SLOT( configure() ) );
	connect( toolbar, SIGNAL( setLibrary() ), SLOT( setLibrary() ) );
	connect( toolbar, SIGNAL( about() ), SLOT( about() ) );
	topLayout->addWidget( toolbar );

	dicttab = new QTabWidget( this );
	dicttab->setFixedWidth( 230 );

	viewer = new VDContentViewer( this );
	viewer->setHelp( vdMakeIndexHelp );
	VDLookupFrame::setWidgets( word = viewer->getWordLabel(), contents );
	connect( viewer, SIGNAL( prev() ), SLOT( historyPrev() ));
	connect( viewer, SIGNAL( next() ), SLOT( historyNext() ));
	connect( viewer, SIGNAL( list() ), SLOT( historyList() ));
	connect( viewer, SIGNAL( edit() ), SLOT( edit() ));
	
	editor = new VDEditor( this );
	connect( editor, SIGNAL( save() ), SLOT( save() ));
	editor->hide();

	QGridLayout *grid = new QGridLayout( topLayout, 0, 0, 4 );
	grid->addMultiCellWidget( dicttab, 0, 9, 0, 0 );
	grid->addMultiCellWidget( viewer, 0, 9, 1, 1 );
	grid->addMultiCellWidget( editor, 0, 9, 1, 1 );
	grid->setColStretch( 0, 2 );
	grid->setColStretch( 1, 4 );
	
	topLayout->activate();

	for( uint i=0; i<dictList.count(); i++ )
	{
		VDictDB *d = dictList.at(i);
		VDLocalLookupFrame *wf = new VDLocalLookupFrame( d, this );
		dicttab->addTab( wf, QString::fromUtf8( d->getDescription() ) );
	}
	setGeometry( Resource::last_x, Resource::last_y, Resource::last_w, Resource::last_h );
	connect( dicttab, SIGNAL( currentChanged( QWidget* ) ), SLOT( changeTab( QWidget* ) ) );
	changeTab( 0 );

	setFont( Resource::defaultFont );
	VDPreferences::setDefaultStyle( Resource::defaultStyle );
	contents->setTextColor( QColor(Resource::defaultColors[2]) );
	about();
}
/*----------------------------------------------------------------------------*/
VD::~VD()
{
	Resource::last_x = x();
	Resource::last_y = y();
	Resource::last_w = width();
	Resource::last_h = height();
	Resource::save();
}
/*----------------------------------------------------------------------------*/
void VD::configure()
{
	VDPreferences vdPreferences;
	vdPreferences.execute();
}
/*----------------------------------------------------------------------------*/
void VD::about()
{
	VDLookupFrame::displayAbout();
}
/*----------------------------------------------------------------------------*/
void VD::edit()
{
	VDLookupFrame *wf = (VDLookupFrame *)dicttab->currentPage();
	if( wf && wf->showEditor( editor ) )
	{
		viewer->hide();
		editor->show();
		dicttab->setEnabled( false );
	}
}
/*----------------------------------------------------------------------------*/
void VD::save()
{
	editor->hide();
	viewer->show();
	dicttab->setEnabled( true );
}
/*----------------------------------------------------------------------------*/
void VD::setLibrary()
{
	VDDatabase vddatabase;
	connect( &vddatabase, SIGNAL( updated(int) ), SLOT( updateTab(int) ) );
	connect( &vddatabase, SIGNAL( deleted(int) ), SLOT( deleteTab(int) ) );
	connect( &vddatabase, SIGNAL( changedOrder(int,int) ), SLOT( moveTab(int,int) ) );
	connect( &vddatabase, SIGNAL( changedInfo(int) ), SLOT( modifyTab(int) ) );
	vddatabase.execute();
}
/*----------------------------------------------------------------------------*/
void VD::updateTab( int t )
{
	VDictDB *d = dictList.at( t );
	VDLocalLookupFrame *wf = new VDLocalLookupFrame( d, this );
	dicttab->addTab( wf, QString::fromUtf8( d->getDescription() ) );
	if( t==0 ) dicttab->setCurrentPage(0);
}
/*----------------------------------------------------------------------------*/
void VD::deleteTab(int t)
{
	dicttab->setCurrentPage( t );
	dicttab->removePage( dicttab->currentPage() );
}
/*----------------------------------------------------------------------------*/
void VD::moveTab(int t, int n)
{
#if QT_VERSION >= 300
	QWidget *p = dicttab->page( t );
#else
	int c = dicttab->currentPageIndex();
	if( c == t ) c = n;
	dicttab->setCurrentPage( t );
	QWidget *p = dicttab->currentPage();
#endif
	dicttab->removePage( p );
	dicttab->insertTab(p, QString::fromUtf8(dictList.at(n)->getDescription()), n);
#if QT_VERSION < 300
	dicttab->setCurrentPage( c );
#endif
}
/*----------------------------------------------------------------------------*/
void VD::modifyTab(int t)
{
#if QT_VERSION >= 300
	QWidget *p = dicttab->page( t );
#else
	int c = dicttab->currentPageIndex();
	if( c != t )
		dicttab->setCurrentPage( t );
	QWidget *p = dicttab->currentPage();
#endif
	if( p ) {
		QString s = QString::fromUtf8( dictList.at(t)->getDescription() );
		dicttab->changeTab( p, s );
	}
#if QT_VERSION < 300
	if( c != t ) dicttab->setCurrentPage( c );
#endif
}
/*----------------------------------------------------------------------------*/
void VD::historyPrev()
{
	VDLookupFrame *wf = (VDLookupFrame *)dicttab->currentPage();
	if( wf ) wf->history( 0 );
}
/*----------------------------------------------------------------------------*/
void VD::historyNext()
{
	VDLookupFrame *wf = (VDLookupFrame *)dicttab->currentPage();
	if( wf ) wf->history( 1 );
}
/*----------------------------------------------------------------------------*/
void VD::historyList()
{
	VDLookupFrame *wf = (VDLookupFrame *)dicttab->currentPage();
	if( wf ) wf->history( 2 );
}
/*----------------------------------------------------------------------------*/
void VD::changeTab( QWidget* )
{
	VDLookupFrame *wf = (VDLookupFrame *)dicttab->currentPage();
	if( wf ) {
		wf->activate();
		setCaption( QString(VD_TITLE) + " - " +
					dicttab->tabLabel( dicttab->currentPage() ) );
	}
}
/*----------------------------------------------------------------------------*/
void VD::setFont( const QFont &font )
{
	QFont f = font;

	contents->setFont( font );
	for( uint i=0; i<dictList.count(); i++ )
		dictList.at(i)->getContainer()->setFont( font );
	f.setBold( true );
	word->setFont( f );
	editor->applyDefaultFont();
}
/*----------------------------------------------------------------------------*/
