/*----------------------------------------------------------------------------*/
/*  vdlocallookupframe.cpp                                                    */
/*----------------------------------------------------------------------------*/
/*  copyright         : (C) 2002 by Dao Hai Lam                               */
/*                      VISC Software & Security Consultant Company           */
/*                      Hall 3, Quang Trung Software City                     */
/*                      Tan Chanh Hiep Ward, District 12,                     */
/*                      Ho Chi Minh city, VIETNAM                             */
/*  website           : http://www.visc-network.com                           */
/*  email             : daohailam at yahoo dot com                            */
/*  last modify       : Sat, 27 Sep 2003 14:31:05 +0700                       */
/*----------------------------------------------------------------------------*/
/*                                                                            */
/*   This program is free software; you can redistribute it and/or modify     */
/*   it under the terms of the GNU General Public License as published by     */
/*   the Free Software Foundation; either version 2 of the License, or        */
/*   (at your option) any later version.                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/
#include <qapp.h>
#include <qframe.h>
#include <vdcontentviewer.h>
#include "vdlocallookupframe.h"
#include "vdrecentdialog.h"
#include "resource.h"
#include "data.h"
/*----------------------------------------------------------------------------*/
#define WINDOW_SIZE		100
#define	MAX_ITEM_IDX	(WINDOW_SIZE-1)
#define vd_min(a, b)	(((a)>(b))?(b):(a))
/*----------------------------------------------------------------------------*/
VDLocalLookupFrame::VDLocalLookupFrame( VDictDB *vdict, QWidget *parent, const char *name ):
						 VDLookupFrame( parent, name )
{
	QFrame *frame = new QFrame( this );
	frame->setFrameShape( QFrame::WinPanel );
	frame->setFrameShadow( QFrame::Sunken );
	QBoxLayout *hbox = new QHBoxLayout( frame, 2, 0 );
	sbar = new QScrollBar( QScrollBar::Vertical, frame );
	list = new VDListBox( frame );
	list->setFrameShape( QFrame::NoFrame );
	list->setHScrollBarMode( QListBox::AlwaysOff );
	list->setVScrollBarMode( QListBox::AlwaysOff );
	hbox->addWidget( list );
	hbox->addWidget( sbar );
	grid->addWidget( frame );
	
	int n = vdict->getWordCount();
	sbar->setMaxValue( n-1 );
	n = vd_min(WINDOW_SIZE, n);
	for( int i=topItem=0; i<n; i++ )
		list->insertItem( QString::fromUtf8(vdict->getWord(i)) );
	list->setCurrentItem( 0 );
	if( vdict->isLatin() )
		edit->setLatinMode();
	setFont( Resource::defaultFont );
	connect( edit, SIGNAL( keyPressed() ), SLOT( lookup() ));
	connect( edit, SIGNAL( browse(int) ), SLOT( browse(int) ));
	connect( edit, SIGNAL( returnPressed() ), SLOT( reference() ));
	connect( list, SIGNAL( selected(int) ), SLOT( reference(int) ));
	connect( list, SIGNAL( highlighted(int) ), SLOT( select(int) ));
	connect( list, SIGNAL( switchFocus(QKeyEvent *) ), edit, SLOT( acceptFocus(QKeyEvent *) ));
	connect( sbar, SIGNAL( valueChanged( int ) ), SLOT( itemChanged( int ) ) );
	vdict->setContainer(this);
	onLookup = false;
	dict = vdict;
}
/*----------------------------------------------------------------------------*/
VDLocalLookupFrame::~VDLocalLookupFrame()
{
	delete dict;
}
/*----------------------------------------------------------------------------*/
void VDLocalLookupFrame::itemChanged( int idx )
{
	int i;
	
	userInput = true;
	if( idx<topItem ) {
		if( idx<topItem-WINDOW_SIZE/2 ) {
			list->clear();
			for( i=idx; i<idx+WINDOW_SIZE; i++ )
				list->insertItem( QString::fromUtf8(dict->getWord(i)) );
		}
		else
		for( i=topItem-1; i>=idx; i-- ) {
			list->removeItem( MAX_ITEM_IDX );
			list->insertItem( QString::fromUtf8(dict->getWord(i)), 0 );
		}
		topItem = idx;
	}
	else
	if( idx>=topItem+WINDOW_SIZE ) {
		if( idx>topItem+WINDOW_SIZE+WINDOW_SIZE/2 ) {
			list->clear();
			for( i=idx-MAX_ITEM_IDX; i<=idx; i++ )
				list->insertItem( QString::fromUtf8(dict->getWord(i)) );
		}
		else
		for( i=topItem+WINDOW_SIZE; i<=idx; i++ ) {
			list->removeItem( 0 );
			list->insertItem( QString::fromUtf8(dict->getWord(i)) );
		}
		topItem = idx - MAX_ITEM_IDX;
	}
	list->setCurrentItem( idx-topItem );
	if( !onLookup ) {
		edit->setText( list->currentText() );
		edit->selectAll();
	}
	userInput = false;
}
/*----------------------------------------------------------------------------*/
// Input into lookup field
void VDLocalLookupFrame::lookup()
{
	onLookup = true;
	sbar->setValue( dict->lookup( edit->text() ) );
	onLookup = false;
}
/*----------------------------------------------------------------------------*/
// Press key up, down, page up, page down
void VDLocalLookupFrame::browse( int code )
{
	switch( code ) {
		case Key_PageUp:
			sbar->subtractPage();
			break;
		case Key_Up:
			sbar->subtractLine();
			break;
		case Key_Down:
			sbar->addLine();
			break;
		case Key_PageDown:
			sbar->addPage();
	}
}
/*----------------------------------------------------------------------------*/
// Click on item in word list
void VDLocalLookupFrame::select( int index )
{
	if( !userInput )
		sbar->setValue( topItem+index );
}
/*----------------------------------------------------------------------------*/
// Press enter on item in word list
void VDLocalLookupFrame::reference()
{
	QString w = list->currentText();

	word->setText( w );
	edit->setText( w );
	edit->selectAll();
	dict->getContents( topItem+list->currentItem(), contents );
	contents->setText( contents->text() );
}
/*----------------------------------------------------------------------------*/
// Double click on item in word list
void VDLocalLookupFrame::reference( int index )
{
	word->setText( list->text( index ) );
	dict->getContents( topItem+index, contents );
	contents->setText( contents->text() );
}
/*----------------------------------------------------------------------------*/
// Click on hyperlink
void VDLocalLookupFrame::reference( const QString &w )
{
	if( !viewer->systemLink( w ) ) {
		int idx = dict->lookup( w );
		dict->getContents( idx, contents );
		contents->setText( contents->text() );
		sbar->setValue( idx );
		word->setText( list->currentText() );
	}
}
/*----------------------------------------------------------------------------*/
void VDLocalLookupFrame::activate()
{
	connect( contents, SIGNAL( switchFocus(QKeyEvent *) ), edit, SLOT( acceptFocus(QKeyEvent *) ));
	connect( contents, SIGNAL( getlink(const QString&) ), SLOT( reference(const QString&) ));
	edit->setFocus();
}
/*----------------------------------------------------------------------------*/
void VDLocalLookupFrame::history( int t )
{
	switch( t ) {
		case 0:
			if( showingAbout )
				dict->getRecent()->last(word, contents);
			else
				dict->getRecent()->prev(word, contents);
			showingAbout = false;
			break;
		case 1:
			if( !showingAbout )
				dict->getRecent()->next(word, contents);
			break;
		case 2:
			if( dict->getRecent()->count() ) {
				VDRecentDialog recent;
				const char *p = recent.show(dict, contents->mapToGlobal( QPoint(1, 1) ));
				if( p!=NULL ) reference( QString( p ) );
			}
			break;
	}
}
/*----------------------------------------------------------------------------*/
bool VDLocalLookupFrame::showEditor( VDEditor *editor )
{
	int idx = list->currentItem();

	if( idx>=0 ) {
		editor->startEdit( dict, topItem+idx  );
		return true;
	}
	return false;
}
/*----------------------------------------------------------------------------*/
